#ifndef FIT_FIT_H
#define FIT_FIT_H

/*
   --- Definitions for the fit subsystem

   $Id: fit.h,v 1.2 1997/12/15 18:51:56 kraemer Exp $
   $Log: fit.h,v $
   Revision 1.2  1997/12/15 18:51:56  kraemer
   no data errors added, fit statistics in function structure

   Revision 1.1.1.1  1997/07/10 17:59:56  kraemer
   revision for cvs

*/

#ifndef EXEC_TYPES_H
#include "exec/types.h"
#endif
#ifndef EXEC_LISTS_H
#include "exec/lists.h"
#endif
#ifndef EXEC_NODES_H
#include "exec/nodes.h"
#endif

#if defined(AMIGA)
#ifndef PROTO_EXEC_H
#include "proto/exec.h"
#endif
#endif

#include "u.h"
#include "fitusr.h"

/*
   --- general flag layout
*/
#define FIT_MASKGLOBAL 0xFF000000     /* global (e.g. debug) */
#define FIT_MASKFUNC   0x00FF0000     /* function specific flags */
#define FIT_MASKQUAL   0x0000FFFF     /* additional qualifiers for multiple functions */

/*
   --- Structures and definitions for graphical output of text 
*/
                                      /* function prototypes */
int fitTxtXFerPut     ( struct List *, const char *, unsigned long );
int fitTxtXFerFree    ( struct List *, unsigned long );
int fitTxtXFerInspect ( struct List *, unsigned long );
/*
   --- Structures and definitions for analyzer & event name transfer
*/

struct SFITANLXFER {
   char *pcAnl, *pcEvt;               /* analyzer/event name */
};

                                      /* function prototypes */
int fitAnlXFerFree    ( struct SFITANLXFER *, unsigned long );
int fitAnlXFerInspect ( struct SFITANLXFER *, unsigned long );
/*
   --- Structures and definitions for data transfer
*/

enum { FITXFER_DATA=0, FITXFER_ERRM, FITXFER_ERRP, FITXFER_MITEM };
#define FITXFER_MDIM 4                /* +1 for data dimension */

struct SFITDATAXFER {
   int nDim;                          /* used dimension (excl. data dim.) */
   unsigned long pl[FITXFER_MDIM+1];  /* data size each dimension */

   double *pd[FITXFER_MDIM+1]
             [FITXFER_MITEM];         /* data each dimension */
};

                                      /* function prototypes */
int fitDataXFerInit    ( struct SFITDATAXFER *, unsigned long );
int fitDataXFerInspect ( struct SFITDATAXFER *, unsigned long );
int fitDataXFerFree    ( struct SFITDATAXFER *, unsigned long );
int fitDataXFerAlloc   ( int [], int, struct SFITDATAXFER *, unsigned long );
/*
   --- Structures and definitions for moments --------------
*/
enum { FITMOM_SUM=0, FITMOM_MEAN, FITMOM_VAR, FITMOM_SKEW, FITMOM_MMOM };
enum { FITMOM_VAL=0, FITMOM_ERRM, FITMOM_ERRP, FITMOM_MITEM };

struct SFITMOM {
   int nn;                            /* number of values considered */
   double dMom[FITMOM_MMOM][FITMOM_MITEM];
};

                                      /* action codes */
enum { FITMOM_DATA=1, FITMOM_FCT, FITMOM_DIFF };

/*
   --- Structures and definitions for statistical information --------------
*/
struct SFITSTAT {
   unsigned long lStatus;             /* condition bits */
   int nDegFree;                      /* # of degrees of freedom */
   double dChisq,                     /* chi**2 */
          dLogLike;                   /* log(likelihood) */
   double dProb;                      /* probability */
   double dLambda;                    /* gradient scaling parameter */
   double dEps[4];                    /* convergence criteria */
   double dTau;                       /* matrix inversion limit */
   long   lL0, lLd;                   /* gradient search parameters */
   long   lEIter;                     /* iterations for error analysis */
};

#define FITSTAT_POISSON   0x00000001
#define FITSTAT_NODATAERR 0x00000002  /* data didn't have own errors */

                                      /* function declarations */
int fitStatProb2Chi( struct SFITSTAT *, unsigned long );
int fitStatChi2Prob( struct SFITSTAT *, unsigned long );
int fitStatInit    ( struct SFITSTAT *, unsigned long );
int fitStatInspect ( struct SFITSTAT *, unsigned long );
/*
   --- Structures for fit window handling ----------------------------------
*/
                                      /* plain fit window structure */
#define MFITWDWDIM 4
struct SFITWDW {
   int nDim;                          /* used # of dimensions (max=4) */
   double dLim[MFITWDWDIM+1][2];      /* +1 for data dimension */
};
                                      /* + node for linked list */
struct SFITWDWNODE {
   struct Node    ln;
   struct SFITWDW sFW;
};

int fitWdwInit   ( struct SFITWDW *, unsigned long );
int fitWdwInspect( struct SFITWDW *, unsigned long );

int fitWdwAnd( struct SFITWDW *, struct SFITWDW *, struct SFITWDW *,
               unsigned long );

int fitWdwListFree   ( struct List *, unsigned long );
int fitWdwListInspect( struct List *, unsigned long );
int fitWdwListDup    ( struct List *, struct List *, unsigned long );

/*
   --- Structure for data calibration --------------------------------------
*/
struct SFITCAL {
   int nCoeff;                        /* # of coefficients (currently=2) */
   double dCoeff[2];
};
                                      /* function prototypes */
int fitCalInit   ( struct SFITCAL *, unsigned long );
int fitCalInspect( struct SFITCAL *, unsigned long );
/*
   --- Structures and definitions for data array
*/
                                      /* structure to hold error info */
struct SFITDATAERROR {
   unsigned long lMode;               /* error mode */
   double dPercent;
};
enum { FITDATAERR_NONE=1, FITDATAERR_OWN, FITDATAERR_STAT,
       FITDATAERR_PERCENT, FITDATAERR_THEORY };

                                      /* structure to hold data+errors */
enum { FITDATA_DATA=0, FITDATA_ERRM, FITDATA_ERRP, FITDATA_MITEM };
#define FITDATA_MDIM 2
#define FITDATA_MAUX 2

struct SFITDATA {
   unsigned long lFlag;
   long lScale[FITDATA_MDIM];         /* gfx scale */
   int nn;
   double dAvgBinsize;
                                      /* x-calibration */
   struct SFITCAL sFCalX;
                                      /* data error descriptors in x,y */
   struct SFITDATAERROR sFDE[FITDATA_MDIM];
                                      /* used data and error arrays
                                         only reference (pdD or pdDS) */
   double *pd [FITDATA_MDIM][FITDATA_MITEM];
                                      /* original data and derived error arrays */
   double *pdD[FITDATA_MDIM][FITDATA_MITEM];
                                      /* original error arrays */
   double *pdO[FITDATA_MDIM][FITDATA_MITEM];
                                      /* scaled data and error arrays */
   double *pdDS[FITDATA_MDIM][FITDATA_MITEM];
                                      /* used function values+errors
                                         only reference (pdFF or pdFS) */
   double *pdF [FITDATA_MITEM];
                                      /* evaluated function values+errors */
   double *pdFF[FITDATA_MITEM];
                                      /* scaled function values+errors */
   double *pdFS[FITDATA_MITEM];
                                      /* used function confidence area
                                         only reference (pdFF or pdFS) */
   double *pdC [FITDATA_MITEM];
                                      /* evaluated confidence area */
   double *pdCC[FITDATA_MITEM];
                                      /* scaled confidence area */
   double *pdCS[FITDATA_MITEM];
                                      /* aux storage for results */
                                      /* used (only reference) */
   double *pdR [FITDATA_MDIM][FITDATA_MITEM];
                                      /* as evaluated */
   double *pdRR[FITDATA_MDIM][FITDATA_MITEM];
                                      /* scaled back */
   double *pdRS[FITDATA_MDIM][FITDATA_MITEM];
                                      /* aux storage, for local use only !*/
   double *pdAux[FITDATA_MAUX];
};
                                      /* coordinate scale types */
enum {
   FITSCALE_LIN=1, FITSCALE_LOG, FITSCALE_SQRT, FITSCALE_RECIP
};
                                      /* Flag bits */
#define FITDATA_REPR    0x00000001    /* fit representation */
#define FITDATA_HISTO   0x00000002    /* fit a histogram */
#define FITDATA_DATALIM 0x00000004    /* fit line between data points */

                                      /* action codes */
enum { FITDATA_SCALEFCT=1, FITDATA_SCALEDIFF };
                                      /* action flags */
#define FITDATA_SCALEERR 0x00000001
                                      /* data structure + linked list handle */
struct SFITDATANODE {
   struct Node    ln;
   struct SFITDATA sFD;
};
                                      /* function prototypes */
int fitDataInit    ( struct SFITDATA *, unsigned long );
int fitDataInspect ( struct SFITDATA *, unsigned long );
int fitDataFree    ( struct SFITDATA *, unsigned long );
int fitDataAlloc   ( int, struct SFITDATA *, unsigned long );
int fitDataScale   ( struct SFITDATA *, long, unsigned long );
int fitDataScaleFct( struct SFITDATA *, unsigned long, unsigned long );
int fitDataErr     ( struct SFITDATA *, long, unsigned long );

int fitDataNetScaled  ( struct SFITDATA *, unsigned long );
int fitDataNetTrue    ( struct SFITDATA *, unsigned long );
int fitDataNetTrueNorm( struct SFITDATA *, unsigned long );

int fitDataNodeFree( struct SFITDATANODE *, unsigned long );
int fitDataNodeListFree( struct List *, unsigned long );

int fitDataWdwList( struct SFITDATA *, struct List *, struct List *, unsigned long );
int fitDataWdwListMerge( struct SFITDATA *, struct List *, struct SFITDATA *, unsigned long );

/*
   ---
*/
                                      /* structure to hold gfx info */
struct SFITGFX {
   unsigned long lFlag;               /* flag bits for gfx routines */
   struct SGH *pGH;                   /* gfx handle */
};

int fitDataGfxLine( struct SFITDATA *, struct SFITGFX *, unsigned long );

                                      /* action flags */
#define FITDATAGFXLINE_CONF 0x00000001

/*
   --- Structure for function derivatives ---------------------------------
*/
struct SFITPARDERIV {
   int np,                            /* # of data points */
       nd;                            /* # of function parameters */
   double *pd;                        /* [nP*nD] function derivatives
                                         D y[*] / D Par
                                      */
};

int fitParDerivAlloc  ( int, int, struct SFITPARDERIV *, unsigned long );
int fitParDerivFree   ( struct SFITPARDERIV *, unsigned long );
int fitParDerivInit   ( struct SFITPARDERIV *, unsigned long );
int fitParDerivInspect( struct SFITPARDERIV *, unsigned long );
int fitParDerivDup    ( struct SFITPARDERIV *, struct SFITPARDERIV *, unsigned long );

/*
   --- Structures for parameter import/export ------------------------------
*/
                                      /* single parameter */
struct SFITPAR {
   double dVal;                       /* parameter value */
   double dRefVal;                    /* guess value
                                         for relatively fixed par.s */
   double dLim[2];                    /* limits */
   double dErr[2];                    /* neg/pos error */
   double dErrSym;                    /* symmetric error */
   double dConfLim[2];                /* neg/pos error, asymmetric */
   long   lRefTyp;                    /* sum/diff/prod/quot active */
   long   lRefNum;                    /* reference parameter number (0..n-1)
                                         for sum/diff/prod/quot */
   unsigned long lFlag;               /* flags */
};
                                      /* flag bits */
#define FITPAR_FIX 0x00000001         /* on/off: parameter fixed/varying */
#define FITPAR_EON 0x00000002         /* on/off: error analysis on/off   */

                                      /* enums */
enum { FITPAR_SUM=1, FITPAR_DIFF=2, FITPAR_PROD=3, FITPAR_QUOT=4 };

int fitParInit   ( struct SFITPAR *, unsigned long );
int fitParInspect( struct SFITPAR *, unsigned long );

                                      /* block of parameters */
struct SFITPARBLK {
   int np;                            /* # of parameters */
   struct SFITPAR *psFP;              /* psFP[np] */
};

int fitParBlkAlloc  ( int, struct SFITPARBLK *, unsigned long );
int fitParBlkFree   ( struct SFITPARBLK *, unsigned long );
int fitParBlkInit   ( struct SFITPARBLK *, unsigned long );
int fitParBlkInspect( struct SFITPARBLK *, unsigned long );

/*
   --- function related structures -----------------------------------------
*/
                                      /* single (partial) function */
struct SFITFCT {
   unsigned long lStatus;             /* for status flag bits */
   unsigned long lFlagNoDup;          /* flags not to be duplicated */
   unsigned long lType;               /* function type (enums below) */
   const char *pcText;                /* descriptive text */
   const char *pcName;                /* function name */
                                      /* function entry point, called */
   int (*eCall) ( void *, double *, double *, double *, double *,
                int, int, unsigned long );
                                      /* function entry point, loaded */
   int (*eLoad)( void *, double *, double *, double *, double *,
                int, int, int );
   dynhandle hDyn;                    /* handle for dynamic function loading */
   void *pvBT,                        /* binary tree for function values */
        *pvBTI;
   int     np;                        /* # of parameters, this function */
   int npSpecial;                     /* special parameter counts:
                                         Gauss,Lortz,PBox : # of params per peak,
                                         exponential polyn: order of exponential
                                      */
   double *pdVal;                     /* parameter values */
   double *pdRefVal;                  /* fixed value for sum/diff/prod/quot*/
   double *pdLim;                     /* [nPar*2] parameter limits */
   double *pdConfLim;                 /* [nPar*2] asymmetric errors */
   unsigned long *plFlag;             /* parameter flags */
   struct SFITUSR sUsr;
   struct SFITPARDERIV sFPD;          /* holds parameter derivatives */
};
                                      /* status flag bits */
#define FITFCT_DEG    0x00000001      /* angles in degree */
#define FITFCT_RAD    0x00000002      /* angles in radian */
#define FITFCT_UNLOAD 0x40000000      /* allow unloading
                                         of dynload()ed stuff */
                                      /* function action flag bits */
                                      /* enable copying of otherwise non-duplicable flag bits */
#define FITFCTDUP_DUPENABLE 0x00000001
                                      /* enums */
enum {
   FITFCT_POLY=1, FITFCT_POLYSQRT, FITFCT_POLYEXP, FITFCT_POLYSQRTEXP,
   FITFCT_EXP, FITFCT_GAUSS, FITFCT_LORENTZ,
   FITFCT_LEGEVEN,
   FITFCT_LEGODD,
   FITFCT_LEG,
   FITFCT_LEGEVENNORM,
   FITFCT_LEGODDNORM,
   FITFCT_LEGNORM,
   FITFCT_GAUSSBOX, FITFCT_LORENTZBOX,
   FITFCT_POI, FITFCT_USR, FITFCT_BTREE
};

/* old 
enum { FITFCT_OFFSPEAKPOS=0, FITFCT_OFFSPEAKWIDTH, FITFCT_OFFSPEAKAREA };
*/
enum { FITFCT_OFFSPEAKAREA=0, FITFCT_OFFSPEAKPOS, FITFCT_OFFSPEAKWIDTH };
#define FITFCT_PARPERPEAK 3
#define FITFCT_PARPERBOX  4
                                      /* function declarations */
int fitbtree( struct SFITFCT *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitusr  ( struct SFITFCT *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitPoly ( void *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitPolySqrt ( void *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitPolyExp ( void *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitPolySqrtExp ( void *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitGauss( void *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitLortz( void *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitExpon( void *,
              double *, double *, double *, double *, int, int, unsigned long );
int fitLegEven    ( void *,
                    double *, double *, double *, double *, int, int, unsigned long );
int fitLegOdd     ( void *,
                    double *, double *, double *, double *, int, int, unsigned long );
int fitLeg        ( void *,
                    double *, double *, double *, double *, int, int, unsigned long );
int fitLegEvenNorm( void *,
                    double *, double *, double *, double *, int, int, unsigned long );
int fitLegOddNorm ( void *,
                    double *, double *, double *, double *, int, int, unsigned long );
int fitLegNorm    ( void *,
                    double *, double *, double *, double *, int, int, unsigned long );

int fitFctAlloc  ( int, struct SFITFCT *, unsigned long );
int fitFctFree   ( struct SFITFCT *, unsigned long );
int fitFctInit   ( struct SFITFCT *, unsigned long );
int fitFctInspect( struct SFITFCT *, unsigned long );

int fitFctDup    ( struct SFITFCT *, struct SFITFCT *, unsigned long );

                                      /* block of functions */
struct SFITFCTBLK {
   unsigned long lStatus;             /* holds flag bits */
   int nf;                            /* # of functions in this block */
   int np;                            /* total # of parameters */
   struct SFITFCT *psFF;              /* [nf] functions */
                                      /* parameter correlation arrays */
   double *pdBeta;                    /* [np] */
   double *pdDiag;                    /* [np] aux. array */
   double *pdAlpha;                   /* [np*np] curvature  matrix */
   double *pdCovar;                   /* [np*np] covariance matrix */
   long   *plRow;                     /* aux array for matrix inv. */
   long   *plCol;                     /* aux array for matrix inv. */
   long   *plRefNum;                  /* parameter cross ref number (0..n-1)*/
   long   *plRefTyp;                  /* parameter cross ref type */
   struct SFITSTAT sFS;               /* misc. statistical info */
};
                                      /* flag bits (status) */
#define FITFCTBLK_KEEP     0x00000001
#define FITFCTBLK_FITDONE  0x00000002
                                      /* flag bits (functions) */
#define FITFCTBLK_POISSON 0x000000001

#define FITFCTBLKPAR_SIGMA 0x00010000 /* FWHM as sigma */
#define FITFCTBLKPAR_AREA  0x00020000 /* parameter output for peak area */
#define FITFCTBLKPAR_POS   0x00040000 /* parameter output for peak pos */
#define FITFCTBLKPAR_WIDTH 0x00080000 /* parameter output for peak width */

                                      /* enable copying of otherwise non-duplicable flag bits */
#define FITFCTBLKFCTPUT_DUPENABLE 0x00010000

                                      /* function declarations */
int fitFctBlkAlloc  ( int, int, struct SFITFCTBLK *, unsigned long );
int fitFctBlkFree   ( struct SFITFCTBLK *, unsigned long );
int fitFctBlkInit   ( struct SFITFCTBLK *, unsigned long );
int fitFctBlkInspect( struct SFITFCTBLK *, unsigned long );
int fitFctBlkDup    ( struct SFITFCTBLK *, struct SFITFCTBLK *, unsigned long );

int fitFctBlkFctPut( struct SFITFCTBLK *, struct SFITFCT *, unsigned long );

int fitFctBlkParGetByNum( struct SFITFCTBLK *, long, struct SFITPAR *, unsigned long );
int fitFctBlkParPutByNum( struct SFITFCTBLK *, long, struct SFITPAR *, unsigned long );

int fitFctBlkParBlkGet( struct SFITFCTBLK *, struct SFITPARBLK *, unsigned long );
int fitFctBlkParBlkPut( struct SFITFCTBLK *, struct SFITPARBLK *, unsigned long );

int fitFctBlkPeak2Par( struct SFITFCTBLK *, long [], int, long [], unsigned long );
int fitFctBlkPar2Peak( struct SFITFCTBLK *, long [], int, long [], unsigned long );
int fitFctBlkPeaks   ( struct SFITFCTBLK *, struct SFITFCTBLK *, unsigned long );
int fitFctBlkByNum   ( struct SFITFCTBLK *, long [], int, struct SFITFCTBLK *, unsigned long );
int fitFctBlkFctDelByNum( struct SFITFCTBLK *, long, unsigned long );

int fitFctBlkCovar   ( struct SFITFCTBLK *, unsigned long );
int fitFctBlkCovarStr( struct SFITFCTBLK *, char *, unsigned long );

int fitFctBlkCorrCopy( struct SFITFCTBLK *, long [][2], int,
                       struct SFITFCTBLK *, unsigned long );

int fitFctBlkParStrByNum( struct SFITFCTBLK *, long, char *, unsigned long );
/*
   --- callback stuff ------------------------------------------------------
*/
                                      /* structure to pass fit display attributes */
struct SFITDISPATTR {
   char *pcColour; int ncColour;
   char *pcLWidth; int ncLWidth;
   char *pcLMode;  int ncLMode;
   char *pcHWidth; int ncHWidth;
   char *pcHMode;  int ncHMode;
};
/*
   --- Common information --------------------------------------------------
*/
struct SFIT {
   unsigned long lStatus;             /* global status flags */
   char *pcWdw;                       /* window name of calling package */
   int (*eFitGfxTxt) ( struct List *, unsigned long );
   int (*eFitStore)  ( struct SFITANLXFER *, struct SFITDATAXFER *, unsigned long );
   int (*eFitLike)   ( struct SFITANLXFER *, struct SFITDATAXFER *, unsigned long );
   int (*eFitError)  ( struct SFITANLXFER *, struct SFITDATAXFER *, unsigned long );
   int (*eFitWdw)    ( char *, int, struct SFITWDW *, unsigned long );
   int (*eFitGfxAttr)( struct SFITDISPATTR *, struct SGFXATTR *, unsigned long );
   struct SFITCAL sFCalX;
   struct SFITDATAERROR sFDE;         /* holds global error mode */
   struct SFITDATA sFD;               /* data as imported from calling package */
   struct SFITDATA sFDFit;            /* data with fit wdws applied, used for fit */
   struct SFITFCTBLK sFFB;            /* block of actually defined functions */
   struct List sListFW;               /* list of fit windows */
   struct SGFXATTR sGA;
/*
   struct SFITSTAT sFS;
*/
};
                                      /* global enums */
enum { FIT_DIMX=0, FIT_DIMY=1, FIT_DIMZ=2 };
                                      /* misc. defines */
enum { FIT_OUTLIST, FIT_OUTPAR, FIT_OUTGFX };

#define MFITLIKEPAR 2
#define MFITLIKEDIM 16                /* default steps for likelihood eval*/
                                      /* command switches */
#define FITCMDSWITCH_PAR   0
#define FITCMDSWITCH_AREA  1
#define FITCMDSWITCH_POS   2
#define FITCMDSWITCH_WIDTH 3
#define FITCMDSWITCH_LIST  0
#define FITCMDSWITCH_SUM   1
                                      /* procedure flags */
#define FITCBACK_LIKE   0x00000001    /* store likelihood in call back */


                                      /* global flag bits */
#define FIT_DEBUG       0x80000000
#define FIT_NOMSG       0x40000000
#define FIT_LORENTZPEAK 0x00000001
#define FIT_POISSON     0x00000002    /* fitmode poisson statistics */

                                      /* functions operating on multiple objects */
int fitDataFctBlkStatNDF( struct SFITDATA *, struct SFITFCTBLK *,
                          struct SFITSTAT *, unsigned long );
int fitWdwPrompt( struct SFITWDW *, struct SFITGFX *, unsigned long );
int fitDataFctNumOfPar( struct SFITDATA *, struct SFITFCT *, unsigned long );
int fitDataFctInitPar ( struct SFITDATA *, struct SFITFCT *, unsigned long );
int fitDataFctBlkValue( struct SFITDATA *, struct SFITFCTBLK *, unsigned long );
int fitDataFctBlkDeriv( struct SFITDATA *, struct SFITFCTBLK *, unsigned long );
int fitDataFctBlkCurv ( struct SFITDATA *, struct SFITFCTBLK *, unsigned long );
int fitDataFctBlkError( struct SFITDATA *, struct SFITFCTBLK *, unsigned long );
int fitDataFctBlkConfLim( struct SFITDATA *, struct SFITFCTBLK *, struct SFITSTAT *, unsigned long );
int fitDataFctBlkConf   ( struct SFITDATA *, struct SFITFCTBLK *, double , unsigned long );

int fitDataWdw    ( struct SFITDATA *, struct SFITWDW *, struct SFITDATA *, unsigned long );
int fitDataWdwMom ( struct SFITDATA *, struct SFITWDW *, struct SFITMOM *,
                    unsigned long,  unsigned long );
int fitDataStatChi ( struct SFITDATA *, struct SFITSTAT *, unsigned long );

                                      /* misc functions */
void fitemsg( const char *, int );
int fitInit( void *, unsigned long );
int fitTerm( void *, unsigned long );
int fit1dlsq( struct SFITDATA *, struct SFITFCTBLK *, struct SFITSTAT *, unsigned long );
int fit1dpoi( struct SFITDATA *, struct SFITFCTBLK *, struct SFITSTAT *, unsigned long );
                                      /* command processors */
void fitCmdDisp( const char *,
                 long *, long *,
                 const char *, const char *, const char *,
                 long *, const char *, double *, long * );
void fitCmdErr (
                 long *, long *, long *, double *,
                 const char *, long *, long * );
void fitCmdExp ( const char *,
                 long *, long *, double *,
                 long * );
void fitCmdFit ( const char *,
                 long *, long *, long *, long *, long *,
                 long *, long *, long *, long * );
void fitCmdInit( long *, long *, long * );
void fitCmdLast( long *, long *, long *, long *, long *, long * );
void fitCmdLeg ( long *,
                 long *, long *, long *, long *, long *, long * );
void fitCmdLike( const char *, const char *,
                 const char *, const char *,
                 const char *,
                 long *, long *);
void fitCmdMLFit(double *, double *, double *, double *, double *,
                 long *, long *, long *, long *, long * );
void fitCmdUsr( const char *, long * );
void fitCmdPar ( const char *,
                 long *, long *, long *, long *, long *,
                 long *, long *, long *, const char *, long *,
                 const char *, const char *, long *, 
                 long *, long *, long *);
void fitCmdPeaks(const char *,
                 long *, long *,
                 double *, double *, double *, double *,
                 long *, long *, long *, long *,
                 long *);
void fitCmdPoly( long *, long *, long *, long *);
void fitCmdRes ( const char *, long *, long *,
                 const char *, const char *, double *,
                 long *, long *, long *, long *);
void fitCmdStore(const char *, const char *, double *,
                 long *, long *, long *, long *);
void fitCmdSum ( const char *,
                 long *, long *, long *,
                 long *, long *, long *,
                 long *, long *, long *,
                 double *,
                 const char *, long *, long * );
void fitCmdWdw ( const char *,
                 long *, long *, long *,
                 long *, long *, long *, long *, long *);
void fitCmdXCal( double *, double *, const char *, const char *,
                 const char *, const char *, long *);

                                      /* error codes */
#undef  LOOSER
#define LOOSER "You loose, you should not use this computer !!"

#define ERRFIT_SYS             -1
#define ERRFIT_INIT            -2
#define ERRFIT_BADADDR         -3
#define ERRFIT_BADDATACOUNT    -4
#define ERRFIT_BADPARCOUNT     -5
#define ERRFIT_BADPARNUM       -6

#define ERRFIT_FCTBTREERESOLVE -7
#define ERRFIT_FCTEVAL         -8

#define ERRFIT_PARCOUNTINCONSISTENT -9
#define ERRFIT_BADPEAKCOUNT        -10
#define ERRFIT_BADPEAKNUM          -11
  
#define ERRFIT_NOPEAKFUNCTION   -12
#define ERRFIT_BADFUNCTIONCOUNT -13
#define ERRFIT_BADFUNCTIONNUM   -14
#define ERRFIT_NOFUNCTION       -15
  
#define ERRFIT_INVERTCURVATURE  -16
  
#define ERRFIT_BADSUBMATRIXCOUNT -17
#define ERRFIT_BADPARREF         -18
  
#define ERRFIT_WDWDIFFDIM        -19

#define ERRFIT_BADDIM            -20
#define ERRFIT_WDWEMPTY          -21

#define ERRFIT_CONVNAN           -22
#define ERRFIT_FEWDATA           -23
#define ERRFIT_CMDPOLYORDER      -24
#define ERRFIT_CMDPARNOTYET      -25
#define ERRFIT_NOPAR             -26
#define ERRFIT_CMDPAREXCL        -27
#define ERRFIT_VALCOUNTMATCHPAR  -28
#define ERRFIT_NODATA            -29
#define ERRFIT_BADWDWLISTSPEC    -30
#define ERRFIT_NOSUCHWDW         -31
#define ERRFIT_NOWDWAVAIL        -32
#define ERRFIT_STATNDF           -33
#define ERRFIT_INVERRMODE        -34
#define ERRFIT_CURSOR            -35
#define ERRFIT_NOGFXAVAIL        -36
#define ERRFIT_BADATTRLISTSPEC   -37
#define ERRFIT_FCTUNKNOWN        -38
#define ERRFIT_NOPEAKSFOUND      -39 
#define ERRFIT_MOMINVACT         -40
#define ERRFIT_SCALEINVACT       -41
#define ERRFIT_XCALFEWCHN        -42
#define ERRFIT_XCALFEWVAL        -43
#define ERRFIT_XCALCHN           -44
#define ERRFIT_XCALSLOPE         -45
#define ERRFIT_XCALFEWERR        -46
#define ERRFIT_XCALFITPOL        -47
#define ERRFIT_FCTUNPHYSICAL     -48
#define ERRFIT_FCTFAILED         -49
#define ERRFIT_FCTDERIVFAILED    -50
#define ERRFIT_FCTUNKNOWNRC      -51
#define ERRFIT_DIVERGENCE        -52
#define ERRFIT_LIKELIHOODFAILED  -53
#define ERRFIT_MATRIXFAILED      -54
#define ERRFIT_PARREFRELATIVE    -55
#define ERRFIT_PARSELFRELATIVE   -56
#define ERRFIT_STORENOTAVAIL     -57
#define ERRFIT_STOREFAILED       -58
#define ERRFIT_LIKEPARTOOMANY    -59
#define ERRFIT_LIKELIMINCONSISTENT -60
#define ERRFIT_LIKENSTEPS        -61
#define ERRFIT_CMDEXPORDER       -62
#define ERRFIT_LSQPOL            -63
#define ERRFIT_CMDLEGORDER       -64
#define ERRFIT_CANTBIND          -65
#define ERRFIT_BTREESYNTAX       -66
#define ERRFIT_BTREERESOLVE      -67
#define ERRFIT_GFXTXTNOTAVAIL    -68
#define ERRFIT_GFXTXTFAILED      -69
#define ERRFIT_IPARINVNAME       -70
#define ERRFIT_IPARCREATEFAILED  -71
#define ERRFIT_CMDNOTYET         -72
#define ERRFIT_IPARINVTYP        -73
#define ERRFIT_IPARREADONLY      -74
#define ERRFIT_IPARINVDIM        -75
#define ERRFIT_IPARINVDIM1       -76
#define ERRFIT_IPARINVDIM2       -77
#define ERRFIT_IPARLOBOUND       -78
#define ERRFIT_IPARHIBOUND       -79
#define ERRFIT_IPARRANGEZERO     -80
#define ERRFIT_BADWDWCOUNT       -81 
#define ERRFIT_ASYMERRNOCONVERGENCE -82
#define ERRFIT_INVCONF           -83
#define ERRFIT_CANTLOAD          -84

#endif


