
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <limits.h>

#include "u.h"
#include "m.h"
#include "cmd.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitcmdpeaks.c,v 1.2 1997/08/29 12:10:06 kraemer Exp $"};

extern struct SFIT sFit;
/*
   --- Command processor for the FPEAKS command

   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitcmdpeaks.c,v $
   Revision 1.2  1997/08/29 12:10:06  kraemer
   gfxattr rework

   Revision 1.1.1.1  1997/07/10 18:00:00  kraemer
   revision for cvs

*/

void fitCmdPeaks(
                 const char *pcNum,
                 long *plGauss, long *plLorentz,
                 double *pdFWHM, double *pdAreaMinimum,
                 double *pdSignificance, double *pdSkewness,
                 long *plPrompt, long *plDel,
                 long *plNoDisp, long *plNoSearch,
                 long *plDebug
                ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__, *pMsg = pDbgMsg;
   unsigned long lFlag = FIT_NOMSG, lFlagG = 0;

   long lPeak, mPeak=0;
   int ip, jp, nn;
   struct Node *psln;
   struct List sLFWN;
   struct SFITWDW     *psFW;
   struct SFITWDWNODE *psFWN;
   struct SFITWDW sFWDef;
   struct SFITFCT sFF;
   struct SFITDATA sFD;
   struct SFITGFX sFGfx;
   struct SGFXATTR sGA;
   unsigned long lType;



#define LN2_8 (0.69*8E0)
#define MMOM 4
   double dMom[MMOM];
   double dd;
   double *pdSmooth, *pdFold;

#define MSTR 132
   char cStr[MSTR+1] = {""};

   double dvLim[2][2];
#define MPEAKITEM 6
   double dvMinPar[ ] = { 1E0, 0E0, 0E0, 0E0, 0E0, 0E0, };
   double *pdPar = NULL;


   if ( *plDebug ) lFlag |= FIT_DEBUG;
   if ( lFlag & FIT_DEBUG ) lFlagG |= G_DEBUG;
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
   memset( &sFF, 0, sizeof(sFF) );
   memset( &sFD, 0, sizeof(sFD) );
   NewList( &sLFWN );
   memset( &sFGfx, 0, sizeof(sFGfx) );
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> gfndwdw...\n" );
   if ( ( sFGfx.pGH = _GFndWdw( sFit.pcWdw ) ) == NULL ) {
/* allow peaks to be defined w/o display
      iRC = ERRFIT_NOGFXAVAIL;
*/
   }
   sFGfx.lFlag = lFlagG;
                                      /* */
   if ( *plDel ) {
      pMsg = "delete"; iRC = ERRFIT_CMDPARNOTYET; goto gError;
   }
                                      /* check if some data defined */
   iRC = ERRFIT_NODATA;
   if ( sFit.sFD.nn > 0 ) {
      if ( sFit.sFD.nn <= FITFCT_PARPERPEAK ) {
         iRC = ERRFIT_FEWDATA; goto gError;
      }
   }
/*
   --- Window handling
*/
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> window handling...\n" );
                                      /* check window interface exists */
   if ( sFit.eFitWdw == NULL ) {
      iRC = ERRFIT_NOWDWAVAIL; goto gError;
   }
                                      /* fetch default limits */
   memset( &sFWDef, 0, sizeof(sFWDef) );
   if ( sFit.eFitWdw( "#", 1, &sFWDef, lFlag ) < 0 ) {
      pMsg = "#";
      iRC = ERRFIT_NOSUCHWDW; goto gError;
   }
   dvMinPar[4] = sFWDef.dLim[0][0];
   dvMinPar[5] = sFWDef.dLim[0][1];
                                      /* establish local window list */
   if ( sFit.sListFW.lh_Head -> ln_Succ ) {
                                      /* duplicate fit window list */
      if ( ( iRC = fitWdwListDup( &sFit.sListFW, &sLFWN, lFlag ) ) != 0 )
         goto gError;
                                      /* ensure inside display region */
      for( psln = sLFWN.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ) {
         psFW = &( ( (struct SFITWDWNODE *) psln ) -> sFW );
         if ( ( iRC = fitWdwAnd( &sFWDef, psFW, psFW, lFlag ) ) != 0 )
            goto gError;
      }
   } else {                           /* at least display region */
                                      /* duplicate window */
      if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }
      psFWN->sFW = sFWDef;
                                      /* link into new list */
      AddTail( &sLFWN, &psFWN->ln ); psFWN = NULL;
   }
/*
   --- apply scaling
*/
   if ( sFit.sFD.nn > 0 ) {
      sFit.sFD.sFCalX = sFit.sFCalX;
      if ( ( iRC = fitDataScale( &sFit.sFD, FIT_DIMX, lFlag ) ) != 0 ) goto gError;
      if ( ( iRC = fitDataScale( &sFit.sFD, FIT_DIMY, lFlag ) ) != 0 ) goto gError;
                                      /* account for background fct */
      if ( (sFit.sFFB.nf > 0) && ( sFit.sFFB.lStatus & FITFCTBLK_KEEP ) ) {
         if ( ( iRC = fitDataFctBlkValue( &sFit.sFD, &sFit.sFFB, lFlag ) ) != 0 ) goto gError;
      } else {
         memset( sFit.sFD.pdF [FITDATA_DATA], 0, sFit.sFD.nn*sizeof(double) );
         memset( sFit.sFD.pdFS[FITDATA_DATA], 0, sFit.sFD.nn*sizeof(double) );
         memset( sFit.sFD.pdFF[FITDATA_DATA], 0, sFit.sFD.nn*sizeof(double) );
      }
      if ( iRC = fitDataNetScaled( &sFit.sFD, lFlag ) ) goto gError;
      if ( iRC = fitDataScaleFct ( &sFit.sFD, FITDATA_SCALEDIFF, lFlag) ) goto gError;
   }
                                      /* type of peak */
   lType = FITFCT_GAUSS;
   if ( sFit.lStatus & FIT_LORENTZPEAK ) lType = FITFCT_LORENTZ;
   if ( *plGauss   ) lType = FITFCT_GAUSS;
   if ( *plLorentz ) lType = FITFCT_LORENTZ;
/*
   --- Determine number of peaks somehow
*/
   while ( *pcNum == ' ' ) pcNum++;
   if ( *pcNum == '*' ) {             /* derive from fit windows */
                                      /* count 'em */
      for ( psln=sLFWN.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ){
         psFWN = (struct SFITWDWNODE *) psln;
         mPeak++;
      }
                                      /* some workspace for max. number */
      if ( ( pdPar = calloc( mPeak, MPEAKITEM*sizeof(double) ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }
                                      /* collect data in wdw */
      if ( sFit.sFD.nn > 0 ) {

         if ( sFit.sFD.nn <= mPeak * FITFCT_PARPERPEAK ) {
            iRC = ERRFIT_FEWDATA; goto gError;
         }

         ip = 0; lPeak = 0;
         for ( psln=sLFWN.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ){

            psFW = & ( ( (struct SFITWDWNODE *) psln ) -> sFW );
            iRC = fitDataWdw( &sFit.sFD, psFW, &sFD, lFlag );
            switch( iRC ) {
            case 0:
                                      /* start values/limits */
               if ( sFD.nn > FITFCT_PARPERPEAK ) {
                  pdPar[ip+4] = psFW -> dLim[0][0];
                  pdPar[ip+5] = psFW -> dLim[0][1];
                  nn = vmom( sFD.pd  [FIT_DIMX][FITDATA_DATA], 1,
                             sFD.pdRR[FIT_DIMY][FITDATA_DATA], 1,
                             sFD.nn,  psFW -> dLim, dMom, MMOM );
                  pdPar[ip+0] = dMom[1];
                  dd = 1E0;
                  if ( nn > 1 ) dd = sqrt( LN2_8 * dMom[2] * (double) nn / (double)(nn-1) );
                  pdPar[ip+1] = dd;
                  pdPar[ip+2] = dMom[0];

                  if ( ( sFD.dAvgBinsize > 0E0 ) &&
                      !( sFD.lFlag & FITDATA_HISTO ) )
                     pdPar[ip+2] *= sFD.dAvgBinsize;

                  ip += MPEAKITEM;
                  lPeak++;
               }
            break;
            case ERRFIT_WDWEMPTY:
            break;
            default: goto gError; break;
            }
            fitDataFree( &sFD, lFlag );
         }
    
         if ( (mPeak = lPeak) <= 0 ) {
            iRC = ERRFIT_NOPEAKSFOUND; goto gError;
         }
      }

   } else {                           /* peaks not by fit windows */
                                      /* max # of peaks */
      if ( c2l( pcNum, &mPeak ) < 0 ) {
         pMsg = (char *) pcNum; iRC=ERRFIT_CONVNAN; goto gError;
      }
      if ( mPeak <= 0 ) {
         pMsg = (char *) pcNum; iRC=ERRFIT_BADPEAKCOUNT; goto gError;
      }
                                      /* some workspace */
      if ( ( pdPar = calloc( mPeak, MPEAKITEM*sizeof(double) ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }

      if ( *plNoSearch ) {            /* no automatic search */
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> nosearch...\n" );
                                      /* some estimates at least */
         if ( sFit.sFD.nn > 0 ) {

            if ( sFit.sFD.nn <= mPeak * FITFCT_PARPERPEAK ) {
               iRC = ERRFIT_FEWDATA; goto gError;
            }

            nn = vmom( sFit.sFD.pd [FIT_DIMX][FITDATA_DATA], 1,
                       sFit.sFD.pdRR[FIT_DIMY][FITDATA_DATA], 1,
                       sFit.sFD.nn, sFWDef.dLim, dMom, MMOM );
            dd = ( sFWDef.dLim[0][1] - sFWDef.dLim[0][0] )
                  / (double) ( mPeak+1 );
            vrampd( sFWDef.dLim[0][0]+dd, sFWDef.dLim[0][1]-dd, 
                    pdPar+0, MPEAKITEM, mPeak );

            if ( ( sFit.sFD.dAvgBinsize > 0E0 ) &&
                 !( sFit.sFD.lFlag & FITDATA_HISTO ) )
               dMom[0] *= sFit.sFD.dAvgBinsize;

            vfill( dMom[0] / (double) mPeak, pdPar+2, MPEAKITEM, mPeak );
            if ( *pdFWHM > 0E0 ) dd = *pdFWHM;
            vfill( dd, pdPar+1, MPEAKITEM, mPeak );
            vfill( dvMinPar[4], pdPar+4, MPEAKITEM, mPeak );
            vfill( dvMinPar[5], pdPar+5, MPEAKITEM, mPeak );
         } else {
            dd = 1E0;
            if ( *pdFWHM > 0E0 ) dd = *pdFWHM;
            vfill( dd, pdPar+1, MPEAKITEM, mPeak );
            dd = 1E0;
            if ( *pdAreaMinimum > 0E0 ) dd = *pdAreaMinimum;
            vfill( dd, pdPar+2, MPEAKITEM, mPeak );
         }
      } else {                        /* automatic search */

         if ( *plPrompt ) {           /* peaks by prompt */
            short  iOld[2] = {0,0};
            double dPnt[2] = {0E0,0E0};
            unsigned long lCursorType = GCURSOR_HAIR;

            if ( sFit.sFD.nn > 0 ) 
               if ( sFit.sFD.nn <= mPeak * FITFCT_PARPERPEAK ) {
                  iRC = ERRFIT_FEWDATA; goto gError;
               }

            lPeak = 0; ip = 0;

            while( lPeak < mPeak ) {
               if ( ( dd = *pdFWHM ) <= 0E0 ) dd = 1E0;
               pdPar[ ip + 1 ] = dd;
               if ( _GWdwIsScreen(sFGfx.pGH) > 0 ) {
                  _GGfxAttrInit( &sGA, lFlagG );
                  _GGfxAttrSet( sFGfx.pGH, &sGA,
                                GGFXATTR_COLOUR | GGFXATTR_FCT |
                                GGFXATTR_LMODE | GGFXATTR_LWIDTH | lFlagG );
                  iRC = _GCursor( sFGfx.pGH, iOld, dPnt, lCursorType, sFGfx.lFlag | G_DONTPLOT | G_COORDLOG );
                  dPnt[1] *= sqrt(PI/(log(2E0)*4E0)) * pdPar[ ip + 1 ];
               } else {               /* terminal mode */
                  nn = _pvrl( "peak position,area", dPnt, 2 );
                  if ( nn >  0 ) iRC = 0;
                  if ( nn == 0 ) iRC = 1;
                  if ( nn <  0 ) iRC = nn;
               }
               if ( iRC < 0 ) { iRC = ERRFIT_CURSOR; goto gError;}
               if ( iRC > 0 ) break;  /* stop ? */
                                      /* accept */
               pdPar[ ip + 0 ] = dPnt[0];
               pdPar[ ip + 2 ] = dPnt[1];
               pdPar[ ip + 4 ] = sFWDef.dLim[0][0];
               pdPar[ ip + 5 ] = sFWDef.dLim[0][1];
               lCursorType |= GCURSOR_OLD;
               lPeak++; ip += MPEAKITEM;
            }
            if ( (mPeak = lPeak) <= 0 ) {
               iRC = ERRFIT_NOPEAKSFOUND; goto gError;
            }

         } else {                     /* no prompt, peak search */
            if ( sFit.sFD.nn <= 0 ) {
               iRC = ERRFIT_NODATA; goto gError;
            }
            if ( sFit.sFD.nn <= mPeak * FITFCT_PARPERPEAK ) {
               iRC = ERRFIT_FEWDATA; goto gError;
            }
                                      /* collect data */
            if ( iRC=fitDataWdwListMerge( &sFit.sFD, &sLFWN, &sFD, lFlag ) )
               goto gError;
                                      /* constraints for peak search */
            dvMinPar[1] = *pdFWHM;
            dvMinPar[2] = *pdAreaMinimum;
            dvMinPar[3] = *pdSignificance * *pdSignificance;
                                      /* search peak */
            if ( ( lPeak = fndpeak(
                                    sFD.pd [FIT_DIMX][FITDATA_DATA],
                                    sFD.pdRR[FIT_DIMY][FITDATA_DATA],
                                    pdSmooth=
                                    sFD.pd[FIT_DIMY][FITDATA_ERRM],
                                    pdFold=
                                    sFD.pd[FIT_DIMY][FITDATA_ERRP],
                                    sFD.nn,
                                    dvMinPar, pdPar, mPeak
                                  ) ) <= 0 ) {
               iRC = ERRFIT_NOPEAKSFOUND; goto gError;
            }
            mPeak = min( mPeak, lPeak );
            sprintf( cStr, "<I> %d peak(s) found, %d peak(s) accepted",
                           lPeak, mPeak );
            _prtcl( cStr, CMDPRTCL_FILE | CMDPRTCL_TERM );
                                      /* determine FWHM from 2nd moments */
            dvLim[1][0] = -DBL_MAX;
            dvLim[1][1] =  DBL_MAX;
            ip = 0;
            for ( lPeak=0; lPeak<mPeak; lPeak++ ) {
               dvLim[0][0] = pdPar[ip+4];
               dvLim[0][1] = pdPar[ip+5];
               nn = vmom( sFD.pd [FIT_DIMX][FITDATA_DATA], 1,
                          sFD.pdRR[FIT_DIMY][FITDATA_DATA], 1,
                          sFD.nn,  dvLim, dMom, MMOM );
               dd = 1E0;
               if ( nn > 1 )
                  dd = sqrt( LN2_8 * dMom[2] * (double) nn / (double)(nn-1) );
               pdPar[ip+1]  = dd;

               if ( ( sFD.dAvgBinsize > 0E0 ) &&
                   !( sFD.lFlag & FITDATA_HISTO ) )
                  pdPar[ip+2] *= sFD.dAvgBinsize;

               ip += MPEAKITEM;
            }
         } /* prompt/noprompt */
      } /* search/nosearch */
   } /* use/don't use fit windows */
/*
   --- sort peaks ascending
*/
   {
   double *pdSort[MPEAKITEM];

   pdSort[0] = pdPar+0;
   pdSort[1] = pdPar+1;
   pdSort[2] = pdPar+2;
   pdSort[3] = pdPar+3;
   pdSort[4] = pdPar+4;
   pdSort[5] = pdPar+5;
   vshsrtd( pdSort, MPEAKITEM, MPEAKITEM, mPeak );
   }
/*
   --- allocate function
*/
   if ( iRC = fitFctAlloc( mPeak*FITFCT_PARPERPEAK, &sFF, lFlag ) ) goto gError;
                                      /* fill entries */
   fitFctInit( &sFF, lFlag);
   switch( sFF.lType = lType ) {
   case FITFCT_GAUSS:
      sprintf( cStr, "Gaussian: %ld peak(s)", mPeak );
      sFF.pcName = strdup("Gaussian");
      sFF.eCall  = fitGauss;
      sFF.npSpecial = FITFCT_PARPERPEAK;
                                      /* remember last type of peak */
      sFit.lStatus &= ~FIT_LORENTZPEAK;
   break;
   case FITFCT_LORENTZ:
      sprintf( cStr, "Lorentzian: %ld peak(s)", mPeak );
      sFF.pcName = strdup("Lorentzian");
      sFF.eCall  = fitLortz;
      sFF.npSpecial = FITFCT_PARPERPEAK;
                                      /* remember last type of peak */
      sFit.lStatus |=  FIT_LORENTZPEAK;
   break;
   default:
      iRC = ERRFIT_FCTUNKNOWN; goto gError;
   break;
   }
   sFF.pcText = strdup(cStr);
                                      /* correct for binsize */
/*
   if ( ( sFit.sFD.dAvgBinsize > 0E0 ) &&
        ( sFit.sFD.lFlag & FITDATA_HISTO ) )
      vsmul( pdPar+2, MPEAKITEM, sFit.sFD.dAvgBinsize, pdPar+2, MPEAKITEM, mPeak );
*/
/*
   --- fill parameter values
*/
   ip = 0; jp = 0;
   for ( lPeak=0; lPeak<mPeak; lPeak++ ){
                                      /* start values/limits */
      sFF.pdLim[(ip+FITFCT_OFFSPEAKPOS)*2+0] = pdPar[jp+4];
      sFF.pdLim[(ip+FITFCT_OFFSPEAKPOS)*2+1] = pdPar[jp+5];
      sFF.pdVal[ ip+FITFCT_OFFSPEAKPOS     ] = pdPar[jp+0];
      sFF.pdVal[ ip+FITFCT_OFFSPEAKWIDTH   ] = pdPar[jp+1];
      sFF.pdVal[ ip+FITFCT_OFFSPEAKAREA    ] = pdPar[jp+2];
      ip += sFF.npSpecial; jp += MPEAKITEM;
   }
                                      /* store parameters */
   if ( iRC = fitFctBlkFctPut( &sFit.sFFB, &sFF, lFlag ) ) goto gError;

                                      /* display peaks */
   if ( (*plNoDisp == 0) && sFGfx.pGH ) {
      unsigned long lFlagG = 0;

      if ( lFlag & FIT_DEBUG ) lFlagG |= G_DEBUG;
      sFGfx.lFlag = lFlagG;

      _GGfxAttrSet( sFGfx.pGH, &sFit.sGA,
                    GGFXATTR_COLOUR | GGFXATTR_FCT |
                    GGFXATTR_LMODE | GGFXATTR_LWIDTH | lFlagG );
                                      /* all function values */
      if ( iRC = fitDataFctBlkValue( &sFit.sFD, &sFit.sFFB, lFlag ) ) goto gError;
                                      /* scale for display */
      if ( iRC = fitDataScaleFct( &sFit.sFD, FITDATA_SCALEFCT, lFlag ) ) goto gError;
                                      /* finall, display it */
      sFGfx.lFlag = lFlagG | G_COORDLOG;
      if ( iRC = fitDataGfxLine ( &sFit.sFD, &sFGfx, lFlag ) ) goto gError;

   }

   iRC = 0;
gError:;
   if ( (iRC < 0) /* && !(lFlag&FIT_NOMSG) */ ) {
      fitemsg( pMsg, iRC );
   }
                                      /* cleanup */
   if (pdPar) free(pdPar); pdPar = NULL;
   fitDataFree( &sFD, lFlag );
   fitFctFree( &sFF, lFlag );
   fitWdwListFree( &sLFWN, lFlag );

if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return;
}

