#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <limits.h>

#include "m.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitcmdpoly.c,v 1.2 1997/12/15 19:04:14 kraemer Exp $"};

extern struct SFIT sFit;
/*
   --- Command processor for the FPOL command

   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitcmdpoly.c,v $
   Revision 1.2  1997/12/15 19:04:14  kraemer
   *** empty log message ***

   Revision 1.1.1.1  1997/07/10 18:00:01  kraemer
   revision for cvs

*/

void fitCmdPoly(
                long *plOrder, long *plExp, long *plSqrt,
                long *plDebug
               ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__, *pMsg = pDbgMsg;
   unsigned long lFlag = 0;

   unsigned long lType = 0;
   int np = 0;
   struct SFITFCT sFF;
   char cStr[80+1];


   if ( *plDebug ) lFlag |= FIT_DEBUG;
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
   memset( &sFF, 0, sizeof(sFF) );
                                      /* check inputs */
   if ( *plOrder < 0 ) {
      iRC = ERRFIT_CMDPOLYORDER; goto gError;
   }
   np = (*plOrder)+1; lType = FITFCT_POLY;
                                      /* */
   if ( *plExp > 0 ) {
/*
      pMsg = "exp";
      iRC = ERRFIT_CMDPARNOTYET; goto gError;
*/
      np += *plExp; lType = FITFCT_POLYEXP;
   }
                                      /* */
   if ( *plSqrt ) {
/*
      pMsg = "sqrt";
      iRC = ERRFIT_CMDPARNOTYET; goto gError;
*/
      lType = FITFCT_POLYSQRT;
      if ( *plExp ) lType = FITFCT_POLYSQRTEXP;
   }
                                      /* allocate local function  */
   if ( ( iRC = fitFctAlloc( np, &sFF, lFlag) ) != 0 ) goto gError;
                                      /* fill entries */
   fitFctInit( &sFF, lFlag);
   switch( sFF.lType = lType ) {
   case FITFCT_POLY:
      sprintf( cStr, "polynomial of order %ld", *plOrder );
      sFF.pcName = strdup("Polynomial");
      sFF.eCall  = fitPoly;
   break;
   case FITFCT_POLYSQRT:
      sprintf( cStr, "sqrt(polynomial of order %ld)", *plOrder );
      sFF.pcName = strdup("sqrt(Polynomial)");
      sFF.eCall  = fitPolySqrt;
   break;
   case FITFCT_POLYEXP:
      sprintf( cStr, "(polynomial of order %ld)*(exponential of order %ld)", *plOrder, *plExp );
      sFF.pcName = strdup("Polynomial*Exponential");
      sFF.eCall  = fitPolyExp;
   break;
   case FITFCT_POLYSQRTEXP:
      sprintf( cStr, "sqrt(polynomial of order %ld)*(exponential of order %ld)", *plOrder, *plExp );
      sFF.pcName = strdup("sqrt(Polynomial)*Exponential");
      sFF.eCall  = fitPolySqrtExp;
   break;
   default:
      iRC = ERRFIT_FCTUNKNOWN; goto gError;
   break;
   }
   if ( ( sFF.pcText = strdup(cStr) ) == NULL ) {
      iRC = ERRFIT_SYS; goto gError;
   }
   sFF.npSpecial = *plExp;
                                      /* initialize parameters */
   fitDataFctInitPar( &sFit.sFD, &sFF, lFlag );
                                      /* add to function block */
   if ( ( iRC = fitFctBlkFctPut( &sFit.sFFB, &sFF, lFlag) ) != 0 ) goto gError;
                                      /*  */

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
   fitFctFree( &sFF, lFlag);
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return;
}
