/*
   Prints an error message
   /mk/  20/11/91
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <errno.h>

#include "u.h"
#include "m.h"
#include "g.h"
#include "fit.h"
static char rcsid[] = {"$Id: fitemsg.c,v 1.2 1997/12/15 18:59:02 kraemer Exp $"};
/* 

   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitemsg.c,v $
   Revision 1.2  1997/12/15 18:59:02  kraemer
   *** empty log message ***

   Revision 1.1.1.1  1997/07/10 18:00:04  kraemer
   revision for cvs

*/
void fitemsg( const char *cTxt, int iErr ) {
 
   static char *cMsg[] = {

   "unknown error",
   "run time system error",
   "fit subsystem initialization failed",
   "bad address",
   "bad data count",
   "bad parameter count",
   "bad parameter number/out of range",

   "can't resolve function",
   "fit function values not calculable",

   "parameter count inconsistent",
   "bad peak count",
   "bad peak number/out of range",

   "no peak functions found",
   "bad function count",
   "bad function number/out of range",
   "no functions found",

   "can't invert curvature matrix",

   "bad submatrix count",
   "bad parameter cross reference/self reference",

   "fit window dimensions are not the same",
   "bad dimension",
   "no data in window",
   "conversion error (not a number)",
   "too few data",
   "invalid order of polynomial",
   "command parameter not yet implemented",
   "no fit function parameters defined yet",
   "command parameters are mutually exclusive",
   "number of values does not match number of function parameters",
   "no data defined",
   "bad window or window list specified",
   "no such window",
   "no windows available at all",
   "ndf<=0 (too many function parameters, too few data points)",
   "invalid error mode",
   "cursor error",
   "no graphics available",
   "bad attribute list specified",
   "unknown function",
   "no peaks found",
   "invalid data source for moment calculation",
   "invalid data source for scaling",
   "too few channel numbers for x-calibration",
   "too few values for x-calibration",
   "invalid channel numbers for x-calibration",
   "slope <= 0 for x-calibration",
   "too few error values for x-calibration",
   "can't fit x-calibration parameters",
   "fit function has entered an unphysical region",
   "fit function not calculable",
   "fit function derivatives not calculable",
   "fit function unknown return code",
   "fit diverges",
   "fit likelihood function not calculable",
   "fit matrix operation failed",
   "reference fit parameter is relatively fixed itself",
   "fit parameter can't be fixed relative to itself",
   "no storing facility available",
   "store failed",
   "too many parameters for likelihood function, must be <="XStr(MFITLIKEPAR),
   "number of parameter limits values inconsistent",
   "invalid # of steps for likelihood evaluation",
   "invalid order of exponential",
   "analytical polynomial fit failed",
   "invalid order of Legendre polynomial",
   "can't bind external function",
   "formula syntax error",
   "formula operands can't be resolved",
   "graphic text output not available",
   "graphic text output failed",
   "invalid global parameter name",
   "global parameter creation failed",
   "command not yet implemented",
   "output global parameter must be of type 'double'",
   "output global parameter is marked as readonly",
   "output global parameter must be a 1- or 2-dimensional array",
   "output global parameter must be a 1-dimensional array",
   "output global parameter must be a 2-dimensional array",
   "output global parameter lower subscript out-of-range",
   "output global parameter upper subscript out-of-range",
   "output global parameter subscripts have zero range",

   "bad window count",
   "no convergence for asymmetric error analysis",
   "invalid confidence limit, must be 0 < conf < 1",
   "can't load external function",
};
#define ERRFIT_MAX ( sizeof(cMsg) / sizeof(char *) )

   int ii;
   int iErrNo = errno;

 
   if ( ( ii = -iErr ) == 0 ) return;
   if ( ii < 0 ) ii = 0;
   if ( ii >= ERRFIT_MAX ) ii = 0;

                                   /* terminal output */
   if ( isatty( fileno(stderr) ) ) {
      if ( iErr == ERRFIT_SYS ) {
         if ( iErrNo != 0 ) {
            fprintf( stderr, "%c[1m<SYS> %s: %s%c[0m\n", 
                             0x1B, cTxt, strerror(iErrNo), 0x1B );
         }
         fflush( stderr );
      } else {
         if ( ii == 0 ) {
            fprintf( stderr, "%c[1m<E> %s: %s %d%c[0m\n", 
                             0x1B, cTxt, cMsg[ii], iErr, 0x1B );
         } else {
            fprintf( stderr, "%c[1m<E> %s: %s%c[0m\n",
                             0x1B, cTxt, cMsg[ii], 0x1B );
         }
      }
   } else {                        /* file output */
      if ( iErr == ERRFIT_SYS ) {
         if ( iErrNo != 0 ) {
            fprintf( stderr, "<SYS> %s: %s\n", 
                             cTxt, strerror(iErrNo) );
         }
         fflush( stderr );
      } else {
         if ( ii == 0 ) {
            fprintf( stderr, "<E> %s: %s %d\n", 
                             cTxt, cMsg[ii], iErr );
         } else {
            fprintf( stderr, "<E> %s: %s\n",
                             cTxt, cMsg[ii] );
         }
      }
   }
   return;
}

