#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>

#include "m.h"
#include "cmd.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitinit.c,v 1.3 2000/04/03 12:58:45 kraemer Exp $"};

struct SFIT sFit;
/*
       fitInit() : initialize fit subsystem
       fitTerm() : terminate  fit subsystem
 
   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitinit.c,v $
   Revision 1.3  2000/04/03 12:58:45  kraemer
   no cmderr(); fpeaks definition fixed

   Revision 1.2  1997/12/15 19:00:30  kraemer
   *** empty log message ***

   Revision 1.1.1.1  1997/07/10 18:00:05  kraemer
   revision for cvs

*/

int fitInit( void *pv,
             unsigned long lFlag
           ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitInit)", *pMsg = pDbgMsg;
   unsigned long lFlagCmd = 0;

   char *pc;
   struct SCMDDEF { char *cCmdDef; void (*eCmdDef)(); unsigned long lFlag; };
   struct SCMDDEF *pCmdDef;

   static struct SCMDDEF sCmdDef[] = {

"FAREA(A(3))|PARAMETERS(T(80)A(2))/\
 FIX(T(0))DIFFERENCE(T(2))SUM(T(2))QUOTIENT(T(2))$DUMMY4(T(2)A(7))\
 VARY(T(0))EON(T(0)A(3))EOFF(T(0)A(3))GUESS(T(50))RELATIVE(T(0))\
 MIN_LIMITS(T(80))MAX_LIMITS(T(80))NOLIMITS(T(0))\
 LIST(T(0))$SWITCH(T(2)D("XStr(FITCMDSWITCH_AREA)"))DEBUG(T(0))", fitCmdPar, 0,

"FDISPLAY(A(2))|FCT(T(80))/SINGLY(T(0))PEAKS(T(0))\
 COLOURS(T(80))WIDTHS(T(80)A(3))LINETYPES(T(80))\
 NPTS(T(2)R(0))WINDOW(T(80)A(1))CONFIDENCE(T(4))DEBUG(T(0))", fitCmdDisp, 0,

"FERRORS(A(2))/NONE(T(0))STATISTICAL(T(0))THEORY(T(0))PERCNT(T(4))\
 ANLID(T(40))OWN(T(0))DEBUG(T(0))", fitCmdErr, 0,

"FEXPONENTIAL(A(3))|NUMBER(T(5)D(1))/\
 PROMPT(T(0))NODISP(A(1)T(0))BACKGROUND(T(4))DEBUG(T(0))", fitCmdExp, 0,

"FIT(A(1))|WINDOWS(T(80))/LEASTSQUARES(A(1)T(0))POISSON(T(0))ZEROES(T(0))\
 ERRANALYSIS(T(0))ITERATIONS(T(2)R(10))TIME(T(2)R(60))\
 LIST(A(2)T(0))NODISPLAY(A(1)T(0))DEBUG(T(0))", fitCmdFit, 0,

"FINIT(A(5))/RELEASE(T(0))NOCONFIRM(T(0))DEBUG(T(0))", fitCmdInit, 0,

"FLAST(A(3))/KEEP(T(0))CLEAR(T(0))DELETE(T(2))LIST(T(0))NOCONFIRM(T(0))\
 DEBUG(T(0))", fitCmdLast, 0,

"FLEGENDRE(A(3))|ORDER(A(2)T(2)R(0))/DEGREES(T(0))RADIAN(T(0))\
 EVEN(T(0))ODD(A(1)T(0))NORMALIZED(T(0))DEBUG(T(0))", fitCmdLeg, 0,

"FLIKELIHOOD(A(4))|ANLID(T("XStr(PATH_MAX)"))EVTID(T("XStr(PATH_MAX)"))/\
 PARAMETERS(T(80))LIMITS(T(80))NSTEPS(T(80))LOGARITHMIC(T(0))DEBUG(T(0))", fitCmdLike, 0,

"FLIST(A(3))|WINDOWS(T(50))/DISPLAY_WINDOWS(T(0))CONDITIONS(T(0))\
 LOOP(T(0))ALL(T(0))GRAPHIC(T(0))\
 NET(T(0))TOTAL(T(0))FWHM(T(0))FITERRORS(T(0))CONFIDENCE(T(4)D(0.683))STORE(T(80))\
 $SWITCH(T(2)D("XStr(FITCMDSWITCH_LIST)"))DEBUG(T(0))", fitCmdSum, 0,

"FMLFIT(A(3))/E1(T(4)D(1E-1))E2(T(4)D(1E-1))E3(T(4)D(1E-5))E4(T(4)D(5E-1))\
 TAU(T(4)D(1E-7))L0(A(2)T(2)D(0))LD(A(2)T(2)D(2))IE(T(2)D(100))\
 LIST(A(1)T(0))DEBUG(T(0))", fitCmdMLFit, 0,

"FMY(A(2)) NAME(T(512))/DEBUG(T(0))", fitCmdUsr, 0,

"FPEAKS(A(3))|NUMBER(A(2)T(8))/GAUSSIAN(T(0))LORENTZIAN(T(0))\
 FWHM(T(4)R(8E0))AREA_MINIMUM(T(4)R(0E0))\
 SIGNIFICANCE(T(4)R(1E0))SKEWNESS(T(4)R(1.5E0))\
 PROMPT(T(0))DELETE(T(0))NODISPLAY(A(3)T(0))NOSEARCH(A(3)T(0))DEBUG(T(0))",
                                                          fitCmdPeaks, 0, 
"fpol|order(t(2)r(0))/exp(t(2)d(0))sqrt(t(0))debug(t(0))", fitCmdPoly, 0,

"FPARAMETERS(A(3))|PARAMETERS(T(80)A(2))/\
 FIX(T(0))DIFFERENCE(T(2))SUM(T(2))QUOTIENT(T(2))PRODUCT(T(2)A(1))\
 VARY(T(0))EON(T(0)A(3))EOFF(T(0)A(3))GUESS(T(50))RELATIVE(T(0))\
 MIN_LIMITS(T(80))MAX_LIMITS(T(80))NOLIMITS(T(0))\
 LIST(T(0))$SWITCH(T(2)D("XStr(FITCMDSWITCH_PAR)"))DEBUG(T(0))", fitCmdPar, 0,

"FPOS(A(3))|PARAMETERS(T(80)A(2))/\
 FIX(T(0))DIFFERENCE(T(2))$DUMMY2(T(2)A(7))QUOTIENT(T(2))$DUMMY4(T(2)A(7))\
 VARY(T(0))EON(T(0)A(3))EOFF(T(0)A(3))GUESS(T(50))RELATIVE(T(0))\
 MIN_LIMITS(T(80))MAX_LIMITS(T(80))NOLIMITS(T(0))\
 LIST(T(0))$SWITCH(T(2)D("XStr(FITCMDSWITCH_POS)"))DEBUG(T(0))", fitCmdPar, 0,

"FRESULT|numbers(T(80))/\
 parameters(t(0))corr(t(0))function(t(256))store(t(80))CONFIDENCE(T(4)D(0.683))\
 graphic(t(0))sigma(t(0))statistics(t(0))debug(t(0))", fitCmdRes, 0,

"FSTORE(A(3))|ANLID(T("XStr(PATH_MAX)"))EVTID(T("XStr(PATH_MAX)"))/\
 CONFIDENCE(T(4)D(0.683))\
 DIFFERENCE(T(0))CHI(T(0))NOCONFIRM(T(0))DEBUG(T(0))", fitCmdStore, 0,

"FSUM (A(3))|WINDOWS(T(50))/DISPLAY_WINDOWS(T(0))CONDITIONS(T(0))\
 LOOP(T(0))ALL(T(0))GRAPHIC(T(0))\
 NET(T(0))TOTAL(T(0))FWHM(T(0))FITERRORS(T(0))CONFIDENCE(T(4)D(0.683))STORE(T(80))\
 $SWITCH(T(2)D("XStr(FITCMDSWITCH_SUM )"))DEBUG(T(0))", fitCmdSum, 0,

"FWIDTH(A(3))|PARAMETERS(T(80)A(2))/\
 FIX(T(0))$DUMMY1(A(7)T(2))$DUMMY2(A(7)T(2))QUOTIENT(T(2))$DUMMY4(T(2)A(7))\
 VARY(T(0))EON(T(0)A(3))EOFF(T(0)A(3))GUESS(T(50))RELATIVE(T(0))\
 MIN_LIMITS(T(80))MAX_LIMITS(T(80))NOLIMITS(T(0))\
 LIST(T(0))$SWITCH(T(2)D("XStr(FITCMDSWITCH_WIDTH)"))DEBUG(T(0))", fitCmdPar, 0,

"FWINDOWS(A(2))|WINDOWS(T(80))/ALL(T(0))DISPLAY_WINDOWS(T(0))CONDITIONS(T(0))\
 LOOP(A(2)T(0))KEEP(T(0))LIST(A(1)T(0))MARK(T(0))DEBUG(T(0))", fitCmdWdw, 0,

"fxcal|offset(t(4)d(0))factor(t(4)d(1))/\
 channels(t(80)d())xnew(t(80)d())errors(t(80)d())anlid(t(18)d())debug(t(0))", fitCmdXCal, 0,
   "", NULL, 0,
   };


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
   if ( lFlag & FIT_DEBUG ) lFlagCmd |= CMD_DEBUG;
                                      /* initialize central structure */
   memset( &sFit, 0, sizeof(sFit) );
/*
   fitStatInit( &sFit. sFS, lFlag );
*/
   fitDataInit( &sFit.sFD, lFlag );
   fitDataInit( &sFit.sFDFit, lFlag );
   fitFctBlkInit( &sFit.sFFB, lFlag );
   fitCalInit ( &sFit.sFCalX, lFlag );
/*
   fitDataErrorInit( &sFit.sFDE, lFlag );
*/
   NewList( &sFit.sListFW );

   pCmdDef = sCmdDef+0;
   while ( pCmdDef->eCmdDef ) {
      if ( ( iRC = _cmddef( pc=pCmdDef->cCmdDef,
                               pCmdDef->eCmdDef,
                               pCmdDef->lFlag | lFlagCmd ) ) != 0 ) {
//         cmderr( pc, iRC );
         iRC = ERRFIT_INIT; goto gError;
      }
      pCmdDef++;
   }

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitTerm( void *pv,
             unsigned long lFlag
           ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitTerm)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* */
   fitWdwListFree( &sFit.sListFW, lFlag );
   fitFctBlkFree( &sFit.sFFB, lFlag );
   fitDataFree( &sFit.sFDFit, lFlag );
   fitDataFree( &sFit.sFD, lFlag );
   if ( sFit.pcWdw ) free( sFit.pcWdw ); sFit.pcWdw = NULL;

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}

