#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>

#include "m.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitpar.c,v 1.1.1.1 1997/07/10 18:00:06 kraemer Exp $"};

/*
   --- Functions to handle parameters and derivatives

       fitParDerivAlloc()   : allocate
       fitParDerivFree()    : free
       fitParDerivInit()    : initialize
       fitParDerivInspect() : inspect output
       fitParDerivDup()     : duplicate
 
       fitParInit()    : initialize
       fitParInspect() : inspect output
 
       fitParBlkAlloc()   : allocate
       fitParBlkFree()    : free
       fitParBlkInit()    : initialize
       fitParBlkInspect() : inspect output
 
   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitpar.c,v $
   Revision 1.1.1.1  1997/07/10 18:00:06  kraemer
   revision for cvs

*/

int fitParDerivDup( struct SFITPARDERIV *psFPDFr,
                    struct SFITPARDERIV *psFPDTo,
                    unsigned long lFlag
                  ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParDerivDup)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFPDFr == NULL ) goto gError;
   if ( psFPDTo == NULL ) goto gError;
   iRC = ERRFIT_BADPARCOUNT;
   if ( psFPDFr->np <= 0 ) goto gError;
   iRC = ERRFIT_BADDATACOUNT;
   if ( psFPDFr->nd <= 0 ) goto gError;
                                      /* allocate space */
   if ( ( psFPDTo -> pd = calloc( psFPDFr->np, psFPDFr->nd*sizeof(double) ) ) == NULL ) {
      iRC = ERRFIT_SYS; goto gError;
   }
                                      /* copy data */
   memcpy( psFPDTo->pd, psFPDFr->pd, psFPDFr->np * psFPDFr->nd * sizeof(double) );
                                      /* remember size */
   psFPDTo -> np = psFPDFr->np;
   psFPDTo -> nd = psFPDFr->nd;

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/

int fitParDerivAlloc( int np,
                      int nd,
                      struct SFITPARDERIV *psFPD,
                      unsigned long lFlag
                    ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParDerivAlloc)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFPD == NULL ) goto gError;
   iRC = ERRFIT_BADPARCOUNT;
   if ( np <= 0 ) goto gError;
   iRC = ERRFIT_BADDATACOUNT;
   if ( nd <= 0 ) goto gError;
                                      /* allocate space */
   if ( ( psFPD -> pd = calloc( np, nd*sizeof(double) ) ) == NULL ) {
      iRC = ERRFIT_SYS; goto gError;
   }
                                      /* remember size */
   psFPD -> np = np;
   psFPD -> nd = nd;

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParDerivFree( struct SFITPARDERIV *psFPD,
                     unsigned long lFlag
                   ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParDerivFree)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFPD == NULL ) goto gError;
   if ( psFPD->pd ) {
      free( psFPD->pd ); psFPD->pd = NULL;
      psFPD->nd = psFPD->np = 0;
   }
   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParDerivInit( struct SFITPARDERIV *psFPD,
                     unsigned long lFlag
                   ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParDerivInit)", *pMsg = pDbgMsg;
   unsigned long ll;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFPD == NULL ) goto gError;
   if ( psFPD->pd ) {
      if ( psFPD->nd > 0 )
         if ( psFPD->np > 0 ) {
            ll = psFPD->nd * psFPD->np;
            memset( psFPD->pd, 0, ll*sizeof(double) );
         }
   }
   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParDerivInspect( struct SFITPARDERIV *psFPD,
                unsigned long lFlag
              ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParDerivInspect)", *pMsg = pDbgMsg;
   int id, ip; double *pd;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFPD == NULL ) goto gError;

   fprintf( stderr, "<D> SFITPARDERIV=%X\n", psFPD );
   fprintf( stderr, "<D> np=%d\n", psFPD->np );
   fprintf( stderr, "<D> nd=%d\n", psFPD->nd );
   fprintf( stderr, "<D> pd=%X\n", psFPD->pd );
   if ( pd=psFPD->pd ) {
      for ( ip=0; ip<psFPD->np; ip++ ) {
         for ( id=0; id<psFPD->nd; id++ ) {
            fprintf( stderr, "<D> deriv[%d][%d]=%G\n", ip, id, *pd ); pd++;
         }
      }
   }

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParInit( struct SFITPAR *psFP,
                unsigned long lFlag
              ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParInit)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFP == NULL ) goto gError;

   memset( psFP, 0, sizeof(*psFP) );
   psFP -> dLim[0] = -DBL_MAX;
   psFP -> dLim[1] =  DBL_MAX;

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParInspect( struct SFITPAR *psFP,
                   unsigned long lFlag
                 ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParInspect)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFP == NULL ) goto gError;

   fprintf( stderr, "<D> SFITPAR=%X\n", psFP );
   fprintf( stderr, "<D> val   =%G\n",  psFP->dVal );
   fprintf( stderr, "<D> refval=%G\n",  psFP->dRefVal );
   fprintf( stderr, "<D> lim=%G %G\n",  psFP->dLim[0], psFP->dLim[1] );
   fprintf( stderr, "<D> reftyp=%ld\n", psFP->lRefTyp );
   fprintf( stderr, "<D> refnum=%ld\n", psFP->lRefNum );
   fprintf( stderr, "<D> flag=%X\n",    psFP->lFlag );

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParBlkAlloc( int np,
                    struct SFITPARBLK *psFPB,
                    unsigned long lFlag
                  ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParBlkAlloc)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );

                                      /* check valid inputs */
   if ( psFPB == NULL ) goto gError;
   iRC = ERRFIT_BADPARCOUNT;
   if ( np <= 0 ) goto gError;
                                      /* allocate space */
   if ( ( psFPB->psFP = calloc( np, sizeof(struct SFITPAR) ) ) == NULL ) {
      iRC = ERRFIT_SYS; goto gError;
   }
                                      /* remember size */
   psFPB -> np = np;

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }

if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParBlkFree( struct SFITPARBLK *psFPB,
                   unsigned long lFlag
                 ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParBlkFree)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFPB == NULL ) goto gError;
   if ( psFPB -> psFP ) {
      free( psFPB->psFP ); psFPB->psFP = NULL;
      psFPB->np = 0;
   }
   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParBlkInit( struct SFITPARBLK *psFPB,
                   unsigned long lFlag
                 ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParBlkInit)", *pMsg = pDbgMsg;
   int ip; struct SFITPAR *psFP;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFPB == NULL ) goto gError;
   if ( psFP=psFPB->psFP ) {
      for ( ip=0; ip<psFPB->np; ip++ ) {
         fitParInit( psFP, lFlag ); psFP++;
      }
   }
   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitParBlkInspect( struct SFITPARBLK *psFPB,
                      unsigned long lFlag
                    ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitParBlkInspect)", *pMsg = pDbgMsg;
   int ip; struct SFITPAR *psFP;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFPB == NULL ) goto gError;

   fprintf( stderr, "<D> SFITPARBLK=%X\n", psFPB );
   fprintf( stderr, "<D> np=%d\n", psFPB->np );
   fprintf( stderr, "<D> psFP=%X\n", psFPB->psFP );
   if ( psFP=psFPB->psFP ) {
      for ( ip=0; ip<psFPB->np; ip++ ) {
         fitParInspect( psFP, lFlag ); psFP++;
      }
   }

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
