#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>

#include "m.h"
#include "g.h"
#include "fit.h"
static char rcsid[] = {"$Id: fitpoly.c,v 1.1.1.1 1997/07/10 18:00:06 kraemer Exp $"};
/*
   --- Polynomial functions

       fitPoly()        : basic polynomial
       fitPolySqrt()    : sqrt(polynomial)
       fitPolyExp()     : polynomial * exponential
       fitPolySqrtExp() : sqrt(polynomial) * exponential

   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
  
       /mk/ /93
   $Log: fitpoly.c,v $
   Revision 1.1.1.1  1997/07/10 18:00:06  kraemer
   revision for cvs


*/
#define DDPAR(ii,jj) ddPar[ (ii) * nX + (jj) ]

/*
   =========================================================================
*/
int fitPoly( void *pvFF,
             double *dPar,                  /* function parameters */
             double *ddPar,                 /* switch=FITFCT_VAL:
                                              ddPar[nPar][0] = lower limit
                                              ddPar[nPar][1] = upper limit
                                              switch=FITFCT_DERIV:
                                              ddPar[nPar][nX] */
             double *dX, double *dY,
             int nPar, int nX,
             unsigned long lFlag
           ) {
   int iRC = FITUSR_OK;
   struct SFITFCT *psFF = pvFF;

   switch( lFlag ) {
                                      /* return # of parameters */
   case FITFCT_NPAR:
      return(nPar);
   break;
                                      /* return start values */
   case FITFCT_VAL:
      memset(  dPar, 0, nPar * sizeof(double) );
   break;
   case FITFCT_FCT:       /* return function values only */
   case FITFCT_DISP:      /* return function values only, for display */
      ddPar = NULL;
   case FITFCT_DERIV:     /* return function values & parameter derivatives  */
      vpoly( dX, 1, dPar, nPar-1, dY, 1, ddPar, nX );
   break;
   default: break;
   }
   return(iRC);
}
/*
   =========================================================================
*/
int fitPolySqrt( void *pvFF,
             double *dPar,                  /* function parameters */
             double *ddPar,                 /* switch=FITFCT_VAL:
                                              ddPar[nPar][0] = lower limit
                                              ddPar[nPar][1] = upper limit
                                              switch=FITFCT_DERIV:
                                              ddPar[nPar][nX] */
             double *dX, double *dY,
             int nPar, int nX,
             unsigned long lFlag
           ) {
   int iRC = FITUSR_OK;
   struct SFITFCT *psFF = pvFF;
   int ip;
   double *pd;

   switch( lFlag ) {
                                      /* return # of parameters */
   case FITFCT_NPAR:
      return(nPar);
   break;
                                      /* return start values */
   case FITFCT_VAL:
      vfill( 1E0, dPar, 1, nPar );
   break;
   case FITFCT_FCT:       /* return function values only */
   case FITFCT_DISP:      /* return function values only, for display */
      ddPar = NULL;
      vpoly( dX, 1, dPar, nPar-1, dY, 1, ddPar, nX );
      if ( vsqrt( dY, 1, dY, 1, nX ) != 0 ) iRC = FITUSR_FCTFAILED;
   break;
   case FITFCT_DERIV:     /* return function values & parameter derivatives  */
      vpoly( dX, 1, dPar, nPar-1, dY, 1, ddPar, nX );
      if ( vsqrt( dY, 1, dY, 1, nX ) != 0 ) iRC = FITUSR_FCTFAILED;
      pd = ddPar;
      for ( ip=0; ip<nPar; ip++ ) {
         if ( vdiv( pd, 1, dY, 1, pd, 1, nX ) != 0 )
            iRC = FITUSR_DERIVFAILED;
         pd += nX;
      }
      vsmul( ddPar, 1, 5E-1, ddPar, 1, nX*nPar );
   break;
   default: break;
   }
   return(iRC);
}
/*
   =========================================================================
*/
int fitPolyExp( void *pvFF,
             double *dPar,                  /* function parameters */
             double *ddPar,                 /* switch=FITFCT_VAL:
                                              ddPar[nPar][0] = lower limit
                                              ddPar[nPar][1] = upper limit
                                              switch=FITFCT_DERIV:
                                              ddPar[nPar][nX] */
             double *dX, double *dY,
             int nPar, int nX,
             unsigned long lFlag
           ) {
   int iRC = FITUSR_OK;
   struct SFITFCT *psFF = pvFF;
   int ip, nParPoly=0, nParExp=0;
   double *pd = NULL, *pdY1, *pdY2;
   double *pdAux = NULL;


   if ( psFF ) {
      nParExp = psFF -> npSpecial;
      nParPoly = nPar - nParExp;
   }

   switch( lFlag ) {
                                      /* return # of parameters */
   case FITFCT_NPAR:
      return(nPar);
   break;
                                      /* return start values */
   case FITFCT_VAL:
/*
      vfill( 1E0, dPar, 1, nPar );
*/
   break;
   case FITFCT_FCT:                   /* return function values only */
   case FITFCT_DISP:      /* return function values only, for display */
      ddPar = pd = NULL;
   case FITFCT_DERIV:                 /* return function values &
                                         parameter derivatives  */
                                      /* allocate aux. storage */
      if ( ( pdAux = calloc(nX,2*sizeof(double)) ) == NULL ) {
         iRC = FITUSR_SYS; goto gError;
      }
      pdY1 = pdAux; pdY2 = pdAux + nX;
                                      /* exponential */
      if ( ddPar ) pd = ddPar + (nParPoly-1)*nX;
      vpoly( dX, 1, dPar+nParPoly-1, nParExp,    pdY2, 1, pd, nX );
                                      /* correct for fudge-added
                                         constant term */
      vsadd( pdY2, 1, -dPar[nParPoly-1], pdY2, 1, nX );
      if ( vexp ( pdY2, 1, pdY2, 1, nX ) != 0 ) iRC = FITUSR_FCTFAILED;
                                      /* polynomial */
      vpoly( dX, 1, dPar           , nParPoly-1, pdY1, 1, ddPar, nX );
                                      /* combine */
      vmul( pdY1, 1, pdY2, 1, dY, 1, nX );
                                      /* derivatives */
      if ( pd=ddPar ) {
         for ( ip=0; ip<nParPoly; ip++ ) {
            vmul( pd, 1, pdY2, 1, pd, 1, nX );
            pd += nX;
         }
         for ( ip=0; ip<nParExp; ip++ ) {
            vmul( pd, 1, pdY2, 1, pd, 1, nX );
            vmul( pd, 1, pdY1, 1, pd, 1, nX );
            pd += nX;
         }
      }
      free(pdAux); pdAux = NULL; /*iRC = FITUSR_NODERIV;*/
   break;

   default: break;
   }
gError:;
   if ( pdAux ) free( pdAux ); pdAux = NULL;
   return(iRC);
}
/*
   =========================================================================
*/
int fitPolySqrtExp( void *pvFF,
             double *dPar,                  /* function parameters */
             double *ddPar,                 /* switch=FITFCT_VAL:
                                              ddPar[nPar][0] = lower limit
                                              ddPar[nPar][1] = upper limit
                                              switch=FITFCT_DERIV:
                                              ddPar[nPar][nX] */
             double *dX, double *dY,
             int nPar, int nX,
             unsigned long lFlag
           ) {
   int iRC = FITUSR_OK;
   struct SFITFCT *psFF = pvFF;
   int ip, nParPoly=0, nParExp=0;
   double *pd = NULL, *pdY1, *pdY2;
   double *pdAux = NULL;


   if ( psFF ) {
      nParExp = psFF -> npSpecial;
      nParPoly = nPar - nParExp;
   }

   switch( lFlag ) {
                                      /* return # of parameters */
   case FITFCT_NPAR:
      return(nPar);
   break;
                                      /* return start values */
   case FITFCT_VAL:
      vfill( 1E0, dPar, 1, nParPoly );
   break;
   case FITFCT_FCT:                   /* return function values only */
   case FITFCT_DISP:      /* return function values only, for display */
      ddPar = pd = NULL;
   case FITFCT_DERIV:                 /* return function values &
                                         parameter derivatives  */
                                      /* allocate aux. storage */
      if ( ( pdAux = calloc(nX,2*sizeof(double)) ) == NULL ) {
         iRC = FITUSR_SYS; goto gError;
      }
      pdY1 = pdAux; pdY2 = pdAux + nX;
                                      /* exponential */
      if ( ddPar ) pd = ddPar + (nParPoly-1)*nX;
      vpoly( dX, 1, dPar+nParPoly-1, nParExp,    pdY2, 1, pd, nX );
                                      /* correct for fudge-added
                                         constant term */
      vsadd( pdY2, 1, -dPar[nParPoly-1], pdY2, 1, nX );
      if ( vexp ( pdY2, 1, pdY2, 1, nX ) != 0 ) iRC = FITUSR_FCTFAILED;
                                      /* polynomial */
      vpoly( dX, 1, dPar           , nParPoly-1, pdY1, 1, ddPar, nX );
      if ( vsqrt( pdY1, 1, pdY1, 1, nX ) != 0 ) iRC = FITUSR_FCTFAILED;
                                      /* combine */
      vmul( pdY1, 1, pdY2, 1, dY, 1, nX );
                                      /* derivatives */
      if ( pd=ddPar ) {
         for ( ip=0; ip<nParPoly; ip++ ) {
            vmul( pd, 1, pdY2, 1, pd, 1, nX );
            if ( vdiv( pd, 1, pdY1, 1, pd, 1, nX ) != 0 ) iRC = FITUSR_DERIVFAILED;
            vsmul( pd, 1, 5E-1, pd, 1, nX );
            pd += nX;
         }
         for ( ip=0; ip<nParExp; ip++ ) {
            vmul( pd, 1, pdY2, 1, pd, 1, nX );
            vmul( pd, 1, pdY1, 1, pd, 1, nX );
            pd += nX;
         }
      }
      free(pdAux); pdAux = NULL; iRC = FITUSR_NODERIV;
   break;

   default: break;
   }
gError:;
   if ( pdAux ) free( pdAux ); pdAux = NULL;
   return(iRC);
}
