#ifndef CMD_H
#define CMD_H
/*
   Definitions for various user interface (command) stuff

   Copyright  1992-1998 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16, all ri

   $Log: cmd.h,v $
   Revision 1.2  2000/04/03 12:35:01  kraemer
   *** empty log message ***

   Revision 1.1.1.1  1999/02/02 20:13:13  kraemer


*/

#include <limits.h>

#if defined(_AIX) || defined(__hpux) || defined(__mips) || defined(sun) || \
    defined(__linux__) || defined(__osf__) || defined(__EMX__)
char *readline( char *);    /* readline from GNU */

#elif defined(MVS) || defined(mvs)
#elif defined(AMIGA)
#else
#error You loose, you should not use this computer
#endif

#include "u.h"

#if defined(__EMX__)
#define INCL_DOSSEMAPHORES
#include <os2.h>
#endif

#include "cmderr.h"
#include "ipar.h"
                                      /* base structure */
struct SCMD {
   unsigned long lStatus;             /* global system status flag */
   int iProtFile;                     /* protocol file handle */
   long mLevel, lLevel;               /* recursion level for EXEC cmd */
   long mCmds,  lCmds;                /* # of commands for EXEC cmd */
   unsigned long lElapsedTime;
   unsigned long lElapsedTimeAccu;
   unsigned long lElapsedBuf[2];
   unsigned long lCPUTime;
   unsigned long lCPUTimeAccu;
   unsigned long lCPUBuf[2];
   char *pcHelpFull;                  /* action for HELP/FULL */
#if defined(AMIGA)
   struct SignalSemaphore sSigSem;    /* for protocol file locking */
#endif
#if defined(__EMX__)
   HMTX hmtx;                         /* for protocol file locking */
#endif
   struct List lh;                    /* command list head */
   struct List lhIPar;                /* IPAR list head */
   struct SCMDERR sCE;                /* holds errors */
   char cProtFile[L_tmpnam+1];        /* protocol file name */
};
                                      /* */
/*
   --- Common status and action flags ----------------------------------------
*/
#define CMDEXEC_MAXLEVEL 10
#define CMDEXEC_MAXCMDS  LONG_MAX

enum { CMD_EXIT=1, CMD_QUIT=2 };
                                      /* */
                                      /* masks */
#define CMD_MASKGLOBAL 0xFF000000     /* global (e.g. debug) */
#define CMD_MASKFUNC   0x00FF0000     /* function specific flags */
#define CMD_MASKQUAL   0x0000FFFF     /* for additional qualifiers */
                                      /* global flags */
#define CMD_DEBUG      0x80000000     /* general check flag */
#define CMD_QUIET      0x40000000     /* suppress error messages */
#define CMD_NOMSG      CMD_QUIET
#define CMD_TIMEON     0x20000000     /* report time consumed */
#define CMD_PRTCL      0x10000000     /* flag to keep protocol */

#define CMD_HIDE       0x00010000     /* mark a "hidden" command */
#define CMD_KEEPPRTCL  0x00020000     /* on terminate: keep protocol */
#define CMD_DONTPRTCL  0x00040000     /* suppress cmd protocol */

                                      /* protocol definitions */
#define CMDPRTCL_NONE    0x00000000
#define CMDPRTCL_TERM    0x00000001   /* output terminal */
#define CMDPRTCL_FILE    0x00000002   /* output file */
#define CMDPRTCL_NOTIME  0x00000100   /* don't include time stamp */
#define CMDPRTCL_NOCOM   0x00000200   /* don't preceed with comment char */

#define MCMD_PAR 160                  /* max # of parameters per command */
/*
   --- function prototypes
*/
int _cmdinit( void *, unsigned long );
int _cmdterm( void *, unsigned long );
int _cmddef ( const char *, int (*)(), unsigned long );
int _cmddec ( const char *, unsigned long );
char *_cmdget( const char *, unsigned long );
int _cmdshell( const char *, unsigned long );

int _cmdError( struct SCMDERR *, unsigned long );

                                      /* command processors */
int _cmdcmdexec( const char *,
                 const char *, const char *, const char *, const char *,
                 const char *, const char *, const char *, const char *,
                 const char *, const char *, const char *,
                 unsigned long *, unsigned long *,
                 unsigned long *, unsigned long *
               );
int _cmdcmdHelp( const char *,
                 const char *, const char *,
                 long *, long *, long *, long *,
                 long *, long *,
                 long *, long *,
                 long *, long *, const char *,
		 long *, const char *, long *,
		 long *, long *, long *,
		 long *, long *
               );
int _cmdcmdexit( unsigned long * );
int _cmdcmdquit( unsigned long * );
int _cmdcmdtime( unsigned long *, unsigned long *, unsigned long * );
int _cmdcmdshell( const char * );

void _prtcl( const char *, unsigned long );
/*
   --- error codes
*/
enum { ERRCMD_SYS=1,
       ERRCMD_BADADDR, ERRCMD_SYNTAX,
       ERRCMD_INIT, ERRCMD_BALANCE, ERRCMD_CONV,
       ERRCMD_CMDITEM, ERRCMD_PARITEM, ERRCMD_NODEABBREV, ERRCMD_NODEADD,
       ERRCMD_ALIAS, ERRCMD_CMDNOTFOUND,
       ERRCMD_BADFCT, ERRCMD_MAXPAR, ERRCMD_CMDNOPAR,
       ERRCMD_DEF2VAL, ERRCMD_VALUE,
       ERRCMD_DUPLICATE, ERRCMD_VAL2DEF, ERRCMD_BADQUOTES,
       ERRCMD_PARTOOMANY, ERRCMD_PARNOTFOUND, ERRCMD_PARREQTOOFEW,
       ERRCMD_PARNOTOPT, ERRCMD_SHELL,
       ERRCMD_EXECLEVEL, ERRCMD_EXECNOFILE, ERRCMD_EXECSYNTAX,
       ERRCMD_EXECNOPROMPT, ERRCMD_EXECJUMPSELECT,
       ERRCMD_EXECMULTIWHEN, ERRCMD_EXECSELECTBALANCE,
       ERRCMD_EXECBADLEVEL,
       ERRCMD_EXECBADSTOP,
};

#endif

