#ifndef IOGOOSY_H
#define IOGOOSY_H
/*
   Definitions for I/O of GOOSY data 
   =================================

   Copyright  (C) 1995-2000 M.Kraemer, GSI Darmstadt 
                             all rights reserved   

   $Log: iogoosy.h,v $
   Revision 1.1  1999/07/17 16:18:59  kraemer
   major overhaul of edas/goosy buffer and swap logic


*/


#include "u.h"
#include "iofh.h"

#define IO_SYSSWAP   0x00000080       /* data swap needed */

                                      /* buffer element header */
struct SIOGOOSYBEH {
   long lDLen;                        /* length of buffer in 16bit words,
                                         without this header */ 
#if defined(U_BIG_ENDIAN)
   short iSubType, iType;             /* buffer subtype/type */ 
#elif defined(U_LITTLE_ENDIAN)
   short iType, iSubType;             /* buffer subtype/type */
#else
#error you loose, unknown endian
#endif
};


                                      /* buffer element header, word oriented */
struct SIOGOOSYBEHW{
   short iDLen;                       /* length of buffer in 16bit words,
                                         without this header */ 
   short iPad; 
   short iType, iSubType;             /* buffer subtype/type */
};


                                      /* event type 3 */
struct SIOGOOSYEVT3 {
   struct SIOGOOSYBEH sBEH;
   long lMode;                        /* compression mode */
   long lLen;                         /* length of uncompressed event */    
//   ... first compressed data longword ...
};				      

                                      /* event type 4, CAMAC single crate */ 
struct SIOGOOSYEVT4 {
   struct SIOGOOSYBEH sBEH;
};				      
                                      /* event type 4,1 (uncompressed) */
struct SIOGOOSYEVT41 {
   struct SIOGOOSYBEH sBEH;
//   ... first data word, second data word ...
};				      
                                      /* event type 4,2 (compressed) */
struct SIOGOOSYEVT42DATA {
   short iID;
   short iData;
};	

struct SIOGOOSYEVT42 {
   struct SIOGOOSYBEH sBEH;
//   ... data ID, data word ...
};	

#if defined(LATER)			      
                                      /*  */
                                      /* SILENA 4418x */
struct SIOSILENA4418{
   unsigned char ucSubEventId;        /* 0..127 identifying subevent 
                                         for following pattern word */
   unsigned char  nDataWord;          /* number of data words following the pattern 
                                         must be equal to the number of pattern
					 bits.
					 ATTENTION: The highest bit is set
                                      */
   short iPattern;                    /* pattern word */            					  
   ... word oriented ...
};
                                      /* event type 1 (SILENA 4418x events) */
struct SIOGOOSYEVT1 {
   struct SIOGOOSYBEHW sBEH;
   ...  SIOSILENA4418 ...
};
#endif	
			      
#if defined(LATER)			      
                                      /* LRS 4300 FERA */
struct SIOLRS4300{
   ... word oriented ...
};
                                      /* event type 5, LRS 4300 FERA */
struct SIOGOOSYEVT5 {
   struct SIOGOOSYBEHW sBEH;
   ...  SIOLRS4300 ...  
};
#endif				      
			      

                                      /* event type 6, subtype = 1, J11, MBD */
struct SIOGOOSYEVT6SUB {
   short iSubEvtLen;                  /* length of subevent excluding header longword */
   unsigned char ucCrate;             /* CAMAC crate number for following data */ 
   unsigned char ucCounter;           /* counter to check correct order of events/subevents */ 
//   ... first data word, second data word ...
};

struct SIOGOOSYEVT6 {
   struct SIOGOOSYBEH sBEH;
// struct SIOGOOSYEVT6SUB sSub[0];    /* */      
};


                                      /* event type 7, subtype=any */
struct SIOGOOSYEVT7 {
   struct SIOGOOSYBEHW sBEH;
//   ... first data word, second data word ...
};


                                      /* CAMAC subevent type 10, subtype=1 */
struct SIOGOOSYEVT101DATA {
#if defined(U_BIG_ENDIAN)
   short iValue, iID;
#elif defined(U_LITTLE_ENDIAN)
   short iID, iValue; 
#else
#error you loose, unknown endian
#endif
};
  
struct SIOGOOSYEVT101SUB {
   struct SIOGOOSYBEH sBEH;
#if defined(U_BIG_ENDIAN)
   unsigned char ucControl,
                 ucCrate;
   short iProcID;                     /*  */
#elif defined(U_LITTLE_ENDIAN)
   short iProcID;                     /*  */
   unsigned char ucCrate,
                 ucControl;
#else
#error you loose, unknown endian
#endif
// struct SIOGOOSYEVT101DATA sData[0];
};


                                      /* FASTBUS subevent type 10, subtype=2 */
                                      /* data word */
struct SIOGOOSYFBUSDATAWORD {
   unsigned bfData    : 12;
   unsigned bfDummy   :  4;
   unsigned bfChannel :  7;
   unsigned bfRange   :  1;
   unsigned bfEvent   :  3;
   unsigned bfGeoAddr :  5;
};				        
                                      /* subevent */  
struct SIOGOOSYEVT102SUB {
   struct SIOGOOSYBEH sBEH;
#if defined(U_BIG_ENDIAN)
   unsigned char ucControl,
                 ucCrate;
   short iProcID;                     /*  */
#elif defined(U_LITTLE_ENDIAN)
   short iProcID;                     /*  */
   unsigned char ucCrate,
                 ucControl;
#else
#error you loose, unknown endian
#endif
                                      /* module header */
#if defined(U_BIG_ENDIAN)
   unsigned char ucLWords;            /* # of data longwords */
   unsigned char ucGeoAddr;
   short iID; 
#elif defined(U_LITTLE_ENDIAN)
   short iID; 
   unsigned char ucGeoAddr;
   unsigned char ucLWords;            /* # of data longwords */
#else
#error you loose, unknown endian
#endif
//   ... first data word, second data word ...
};                                     
                                      /* event type 10, VME */
struct SIOGOOSYEVT10 {
   struct SIOGOOSYBEH sBEH;
#if defined(U_BIG_ENDIAN)
   short iTrigger, iPad;              /*  */ 
#elif defined(U_LITTLE_ENDIAN)
   short iPad, iTrigger;              /*  */
#else
#error you loose, unknown endian
#endif
   long lCount;                       /* event counter */ 
//   ... subevent 1, subevent 2, ...
};

                                      /* timestamp, subtype=0 */
struct SIOGOOSYTIMESTAMP {
   struct SIOGOOSYBEH sBEH;
   char cDate[24];
};

                                      /* GOOSY data element (not yet) */
                                      /* these buffer elements may appear in any buffer */
enum {
   IOGOOSY_TIMESTAMP=9000,            /* time stamp */
   IOGOOSY_CAMACREADOUT=2001,         /* CAMAC read-out table,
                                         subtype==1: init
					 subtype==2: readout
					 subtype==3: reset
                                      */
   IOGOOSY_FBUSREADOUT=2002,          /* Fastbus read-out table,
                                         subtype==1: init
                                      */
   IOGOOSY_VMEREADOUT=2003,           /* CAMAC read-out table,
                                         subtype==1: init
                                      */
};				      
/* 
   --- GOOSY buffer structures 
*/
                                      /* buffer header */				      
struct SIOGOOSYBH {
   long lDLen;                        /* length of buffer in 16bit words,
                                         without this header */ 
#if defined(U_BIG_ENDIAN)
   short iSubType, iType;             /* buffer subtype/type */ 
   unsigned char ucBeg,               /* =1: fragment starts at buffer end */   
                 ucEnd;               /* =1: fragment ends at buffer start */ 
   short iUsed;                       /* # of 16bit words 
                                         actually following this header */
#elif defined(U_LITTLE_ENDIAN)
   short iType, iSubType;             /* buffer subtype/type */
   short iUsed;                       /* # of 16bit words 
                                         actually following this header */
   unsigned char ucEnd,               /* =1: fragment ends at buffer start */                 
                 ucBeg;               /* =1: fragment starts at buffer end */                            
#else
#error you loose, unknown endian
#endif
   long lBuf;                         /* current buffer number (of same type) */
   long lEvt;                         /* current # of events/fragments */
   long lCurrent;                     /* index of last processed buffer element
                                         on tape/disk: = 0/1
					 for lmdi-converted data (IBM/MVS): -1
                                      */
   unsigned long ulTime[2];           /* VMS time stamp */
   unsigned long ulByteSex;           /* 1: OK, else: longword swap */
   long lLastEventLen;                /* total event length,
                                         in case of a fragment at the end */
   unsigned long ulFree[2];           /* reserved */

};
                                      /* buffer header, word aligned */				      
struct SIOGOOSYBHW {
   short iDLen;                       /* length of buffer in 16bit words,
                                         without this header */ 
   short iPad; 
   short iType, iSubType;             /* buffer subtype/type */
   short iUsed;                       /* # of 16bit words 
                                         actually following this header */
   unsigned char ucEnd,               /* =1: fragment ends at buffer start */                 
                 ucBeg;               /* =1: fragment starts at buffer end */                            
   long lBuf;                         /* current buffer number (of same type) */
   long lEvt;                         /* current # of events/fragments */
   long lCurrent;                     /* index of last processed buffer element
                                         on tape/disk: = 0/1
					 for lmdi-converted data (IBM/MVS): -1
                                      */
   unsigned long ulTime[2];           /* VMS time stamp */
   unsigned long ulByteSex;           /* 1: OK, else: longword swap */
   long lLastEventLen;                /* total event length,
                                         in case of a fragment at the end */
   unsigned long ulFree[2];           /* reserved */

};
                                      /* buffer types */
enum { 
   IOGOOSY_FILEHEAD=2000,             /* file header, subtype==1 */
   IOGOOSY_ACKNOWLEDGE=3000,          /* acknowledge:
                                         no data, end of buffer stream, 
					 subtype==1 */
   IOGOOSY_MBDNONSTD=1,               /* nonstandard buffer , subtype==1 */ 
   IOGOOSY_J11SILENA=2,               /* J11-generated Silena events , 
                                         subtype==1 */ 
   IOGOOSY_BE3=3,                     /* buffer elements of type 3, 
                                         subtype==1: compress mode 1 
                                         subtype==2: compress mode 2 
			              */ 
   IOGOOSY_BE4=4,                     /* buffer elements of type 4, 
                                         subtype==1: uncompressed 
                                         subtype==2: compressed 
			              */ 
   IOGOOSY_STDFERA=5,                 /* LRS FERA events, 
                                         with standard headers,
					 subtype==1 */ 
   IOGOOSY_MBDSTD=6,                  /* MBD standard, J11-defined events, subtype==1 */ 
   IOGOOSY_MBDUSR=7,                  /* MBD standard, J11-defined user structures
                                         subtype==any */ 
   IOGOOSY_VME=10,                    /* VME events, subtype==1 */
   IOGOOSY_SILENA=12,                 /* SILENA formatted events, 
                                         no standard header, 
					 subtype==1
				      */	  
   IOGOOSY_FERA=15,                   /* FERA formatted events,
                                         no standard header,
					 subtype==1
				      */
   IOGOOSY_DE=1000,                   /* GOOSY data element,
                                         subtype==1: spectrum 
                                         subtype==2: condition
                                         subtype==3: picture
                                         subtype==4: polygon
                                         subtype==5: calibration
                                         subtype==6: data element
                                      */ 
   IOGOOSY_MAINZ=10101,               /* external user buffer, Mainz,
                                         subtype==any */				      				       	 
   IOGOOSY_THD=10102,                 /* external user buffer, THD,
                                         subtype==any */				      				       	 
   IOGOOSY_CAVEB=10103,               /* external user buffer, CAVEB,
                                         subtype==any */				      				       	 
};

                                      /* file header */
struct SIOGOOSYFH {
   struct SIOGOOSYBH sBH;
   short iLabel; char cLabel[30];
   short iFile;  char cFile[86];
   short iUser;  char cUser[30];
                 char cTime[24];
   short iRun;   char cRun[66];
   short iExp;   char cExp[66];
   long  lCom;                        /* actual # of 78-char comment lines following */
   struct {
      short nCom;
      char cCom[78];
   } sCom[46];

};				      				      				      
                                      /*  */
//#define IOFRAGMENT_ATBEG 0x00000001				      
//#define IOFRAGMENT_ATEND 0x00000002				      
                                      /* function prototypes */
int _iogoosybh    ( struct SIOGOOSYBH *, unsigned long );
void *_iogoosydefrag( struct SIOGOOSYBEH *, void *, unsigned long );
int _iogoosy1000x ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy2000  ( struct SIOGOOSYFH *, struct SIOFH *, unsigned long );
int _iogoosy3000x ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy01x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy02x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy03x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy04x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy05x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy06x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy07x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy10x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy12x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy15x   ( struct SIOGOOSYBH *, unsigned long );
int _iogoosy10101x( struct SIOGOOSYBH *, unsigned long );
int _iogoosy10102x( struct SIOGOOSYBH *, unsigned long );
int _iogoosy10103x( struct SIOGOOSYBH *, unsigned long );

#endif
